%Function Create_Regressor_DH_blackOut.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load central banks debt holdings and auction data from parent directory and
%         create a debt purchases variable.
%
%Inputs:    - aAUC.mat           Matlab-file with auction data.
%           - aDH_monthly.mat    Matlab-file with central bank debt holdings data.
%
%Outputs:   - aDH_interp_blackout_diff.mat Matlab-file containing  CB net asset purchases.
close all; clear;clc;

%BASED ON THE 10Y MATURITY BUCKET.
load('aDH_monthly.mat');
load('aAUC.mat');
iBlackoutDays = 1; %so one day before is excluded (and auction day ofcourse).


%Countries and Days.
aCountries = aAUC(1,:);
vDates_day = aAUC{3,1}(:,1);

%Extrapolate holdings data to obtain a entry for the first day of January
%1999. This observation is needed to obtain data for the first month.
%GREECE IS PROBLEMATIC (ENTERED EURO IN 2001).
vDebtSecHoldStart = zeros(1,numel(aCountries));
for i = 1:numel(aCountries)
    vDebtSecHoldStart(i) = interp1(aDH_monthly{2,i}(:,1), aDH_monthly{2,i}(:,2),vDates_day(1),'pchip','extrap');
end

%Create .mat file with the interpolated data and one with the changes. Note
%that taking mere differences of the interpolated series would lead to a
%wrong series.
aDH_interp_blackout_diff = cell(2,numel(aCountries));
for i = 1:numel(aCountries)
    aDH_interp_blackout_diff{1,i} = aCountries(i);
    vDH_interp_blackout_diff      = zeros(length(vDates_day),1);
    vAuctions                     = aAUC{3,i}(:,2);
    iLoc_1                        = find(vDates_day>datenum('31-12-1998', 'dd-mm-yyyy'),1,'first');
    for t=1:length(aDH_monthly{2,1}(:,1))
        if t~=1
            iLoc_1 = find(vDates_day>aDH_monthly{2,1}(t-1,1),1,'first');
        end
        iLoc_2 = find(vDates_day<=aDH_monthly{2,1}(t,1),1,'last');
        vAuctionDays = vAuctions(iLoc_1:iLoc_2);
        iAuctionDays = sum(vAuctionDays);
        
        %BlackOutDays.
        vAuction_Indices = find(vAuctionDays);
        vBlackOutIndices = vAuction_Indices;
        if ~isempty(vAuction_Indices)
            for l=0:iBlackoutDays
                vBlackOutIndices = [vBlackOutIndices;vAuction_Indices-l]; %#ok<AGROW>
            end
        end
        vBlackOutIndices = unique(vBlackOutIndices);
        %vBlackOutIndices(vBlackOutIndices==0)=[];
        %vBlackOutIndices(vBlackOutIndices>length(vAuctions))=[];
        if t~=1
            if(~isempty(vBlackOutIndices(vBlackOutIndices==0)))%a blackout day falls in previous month
                vDH_interp_blackout_diff(vInterDays_prev) = vDH_interp_blackout_diff(vInterDays_prev) + vDH_interp_blackout_diff(iLoc_2_prev)/length(vInterDays_prev); %redistribute the budget over the other days (exclude previous auction days.
                vDH_interp_blackout_diff(iLoc_2_prev) =0; %blackout day.
            end
        end
        
        vBlackOutDays = (iLoc_1-1)+vBlackOutIndices;
        vInterDays = setdiff(iLoc_1:iLoc_2,vBlackOutDays);
        iIntrvDays = length(vInterDays);
        
        if t==1
            dBudget = (aDH_monthly{2,i}(t,2)-vDebtSecHoldStart(i))/iIntrvDays;
            vDH_interp_blackout_diff(vInterDays) = dBudget;
        else
            dBudget = (aDH_monthly{2,i}(t,2)-aDH_monthly{2,i}(t-1,2))/iIntrvDays;
            vDH_interp_blackout_diff(vInterDays) = dBudget;
        end
        %iLoc_1_prev = iLoc_1;
        iLoc_2_prev = iLoc_2;
        vInterDays_prev = vInterDays;
    end
    %Transform data to billions, the scale of the coefficient was so
    %small since it was in millions. makes more sense to look at billions.
    aDH_interp_blackout_diff{2,i} = [vDates_day,vDH_interp_blackout_diff/1000];
end
save('aDH_interp_blackout_diff', 'aDH_interp_blackout_diff');

%Plot Data
aDH_interp_blackout_diff = aDH_interp_blackout_diff(:,[1:3,5:7,9,11:12]);
aCountries = string(aDH_interp_blackout_diff(1,:));
mDH      =[];
mDH_diff =[];
figure
hold on
for i =1:numel(aCountries)
    mDH      = [mDH,aDH_interp_blackout_diff{2,i}(:,2)]; %#ok<AGROW>
    mDH_diff = [mDH_diff,aDH_interp_blackout_diff{2,i}(:,2)]; %#ok<AGROW>
    plot(aDH_interp_blackout_diff{2,i}(:,1),aDH_interp_blackout_diff{2,i}(:,2))
    datetick('x' ,'mm-yyyy')
    xlim([aDH_interp_blackout_diff{2,i}(1,1), aDH_interp_blackout_diff{2,i}(end,1)])
    ylabel('euro (billions)')
end
legend(aCountries, 'location', 'northwest')
%title('EMU Government Debt Holdings')
hold off

figure
plot(vDates_day,mDH_diff)
datetick('x' ,'mm-yyyy')
xlim([aDH_interp_blackout_diff{2,i}(1,1), aDH_interp_blackout_diff{2,i}(end,1)])
ylabel('euro (billions)')
legend(aCountries, 'location', 'northwest')
title('\Delta EMU Government Debt Holdings')

figure
plot(vDates_day,max(mDH_diff,0))
datetick('x' ,'mm-yyyy')
xlim([aDH_interp_blackout_diff{2,i}(1,1), aDH_interp_blackout_diff{2,i}(end,1)])
ylabel('euro (billions)')
%xlim([vDates_day(1), vDates_day(end)])
legend(aCountries, 'location', 'northwest')
%title('max(\Delta EMU Government Debt Holdings,0)')


%Zoom in on QE period
iLoc_QE = find(vDates_day>=datenum('02-07-2009','dd-mm-yyyy'),1,'first');
figure
plot(vDates_day(iLoc_QE:end),mDH_diff(iLoc_QE:end,:))
datetick('x' ,'mm-yyyy')
xlim([aDH_interp_blackout_diff{2,i}(iLoc_QE,1), aDH_interp_blackout_diff{2,i}(end,1)])
ylabel('euro (billions)')
legend(aCountries, 'location', 'northwest')
title('\Delta EMU Government Debt Holdings, QE Period')

figure
plot(vDates_day(iLoc_QE:end),max(mDH_diff(iLoc_QE:end,:),0))
datetick('x' ,'mm-yyyy')
xlim([aDH_interp_blackout_diff{2,i}(iLoc_QE,1), aDH_interp_blackout_diff{2,i}(end,1)])
ylabel('euro (billions)')
%xlim([vDates_day(1), vDates_day(end)])
legend(aCountries, 'location', 'northwest')
%title('max(\Delta EMU Government Debt Holdings,0), QE Period')



