%Function Create_Regressor_DH_blackOut_hat.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load central banks debt holdings and auction data (and other covariates) from parent directory and
%         create debt purchases variable, instrumental variable approach.
%         (robustness)
%
%Inputs:    - aAUC.mat           Matlab-file with auction data.
%           - aDH_monthly.mat    Matlab-file with central bank debt holdings data.
%
%Outputs:   - aDH_interp_blackout_diff_hat.mat Matlab-file containing  CB net asset purchases.
clear;clc;
%load('aDH_interp', 'aDH_interp');
load('aDH_interp_blackout_diff');
load("../Data/Yields/aYields_change.mat")
load("../Data/Market Volatility/Factor Analysis/aYields_Vola_Factor_individual.mat")
load("aAUC.mat")
load("../Data/Market Conditions/aFinMarket.mat")

%Specify instrument lag.
iLag    = 22;

aCountries  = aAUC(1,:);
iN          = numel(aCountries);
vDates      = aDH_interp_blackout_diff{2,1}(:,1);

%Specify which countries to include in the regression per maturity.
[~,aCountrySet,~]    = xlsread("../Data/CountrySetPerMaturity.xlsx","B1:Z3");
[~,aSetCntry,~]      = intersect(aCountries,aCountrySet(2,:));   %10-Year maturity.

aDH_interp_blackout_diff_hat  = cell(2,12);
aAUC_10Y             = aAUC([1,3],aSetCntry);
aYields_Vola_10Y     = aYields_Vola_Factor_individual([1,3],aSetCntry);

mAUC  = zeros( size(aAUC_10Y{2,1}(:,1),1) ,size(aAUC_10Y,2));
mVOLA = zeros( size(aYields_Vola_10Y{2,1}(:,1),1) ,size(aYields_Vola_10Y,2));
for i=1:size(aAUC_10Y,2)
    mAUC(:,i)  = aAUC_10Y{2,i}(:,2);
    mVOLA(:,i) = aYields_Vola_10Y{2,i}(:,2);
end

mX_AUC= zeros(size(mAUC,1),40);
for i =1:size(aAUC_10Y,2)
    for t= 6 : size(mAUC,1)-4
        vRange      = (t+4:-1:t-5)';
        vX_dom      = mAUC(vRange,i)';
        vX_dom_vol  = mAUC(vRange,i)'.*mVOLA(t,i);
        vX_for      = sum(mAUC(vRange,1:end~=i),2)';
        vX_for_vol  = sum(mAUC(vRange,1:end~=i).*mVOLA(t,1:end~=i),2)';
        vX_AUC      = [vX_dom, vX_dom_vol, vX_for, vX_for_vol];
        mX_AUC(t,:) = vX_AUC; 
    end
end

for i=1:iN
    aDH_interp_blackout_diff_hat{1,i} = aCountries{i};
    %vY      = aDH_interp_diff{2,i}(23:end,2);
    %vX      = aDH_interp_diff{2,i}(1:end-22,2); 
   
    vY      = max(aDH_interp_blackout_diff{2,i}(iLag+1:end,2),0);
    vX      = max(aDH_interp_blackout_diff{2,i}(1:end-iLag,2),0); %average number of working days in month is 21.74
    
    
    mX_FM   = [aFinMarket{2,1}(iLag:end-1,2),log(aFinMarket{2,2}(iLag:end-1,2)),... %begin index = ilag +1 -1. 
                log(aFinMarket{2,3}(iLag:end-1,2)),log(aFinMarket{2,4}(iLag:end-1,2))];
    
    %mX_for  =
    mX      = [ones(length(vX),1),vX,mX_FM, mX_AUC(iLag+1:end,:)];
    vBeta   = (mX'*mX)\mX'*vY;
    vY_hat  = mX*vBeta;
    aDH_interp_blackout_diff_hat{2,i} = [vDates,[zeros(22,1)*nan;vY_hat]];
    disp(aCountries{i})
    disp('vBeta')
    disp(vBeta)
    disp('corr')
    disp(corr(vY,mX(:,2)))
    figure
    plot( repmat(vDates(23:end),[1,2]), [vY,vY_hat])
    datetick('x' ,'mm-yyyy')
    xlim([vDates(23), vDates(end)])
    legend('True','Predicted','Location','northwest')
    ylabel('euro (billions)')
    title(aCountries{i})
end
save('aDH_interp_blackout_diff_hat.mat', 'aDH_interp_blackout_diff_hat')

mDH_diff       =[];
for i =1:iN
    mDH_diff   = [mDH_diff,aDH_interp_blackout_diff{2,i}(:,2)]; %#ok<AGROW> 
end
iDisplay_Start = 1;
figure
plot(repmat(vDates(iDisplay_Start:end),[1 iN]),max(mDH_diff(iDisplay_Start:end,:),0))
datetick('x' ,'mm-yyyy')
xlim([vDates(iDisplay_Start), vDates(end)])
legend(string(aCountries),'Location','northwest')
ylabel('euro (billions)')
%title('QE spending (interpolated)')
