%Function Create_Regressor_FloatingSupply.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load central banks purchases data from parent directory and
%         create floating supply variable.
%
%Inputs:    - aDH_interp_blackout_diff.mat    Matlab-file with central bank debt purchases data.
%
%Outputs:   - aMktDepth.mat       Matlab-file containing floating supply data.
close all; clear;clc;

%BASED ON THE 10Y MATURITY BUCKET.
load('aDH_interp_blackout_diff.mat');


%Countries and Days.
aCountries = aDH_interp_blackout_diff(1,:);
vDates_day = aDH_interp_blackout_diff{2,1}(:,1);
iN         = numel(aCountries);
%Note that Greece should be on the right location in the file (I denote as
%"EL" instead of "GR")
mGovDebt       = xlsread('../Data/Government Debt/GovDebt_maturity_geq_1y.xlsx', 'B6:M90');
vDates_GovDebt = x2mdate(xlsread('../Data/Government Debt/GovDebt_maturity_geq_1y.xlsx', 'A6:A90'));

mGovDebt    = mGovDebt/1000; %Transform data to billions.

%As for some countries these data are missing, extrapolate holdings data 
%to obtain a entry for 1999.
for i = setdiff((1:iN),[2,5,7,9,])
    mGovDebt(1:4,i) = interp1(vDates_GovDebt(5:end), mGovDebt(5:end,i),vDates_GovDebt(1:4),'linear','extrap');
end
%figure works.
%plot(repmat(vDates_GovDebt, [1 iN]), mGovDebt)
%datetick('x','mm-yyyy')
%legend(string(aCountries), 'location', 'northwest')
%xlim([vDates_GovDebt(1), vDates_GovDebt(end)])
%ylabel('euro (billions)')

%Now obtain daily data
aGovDebt = cell(2,numel(aCountries));
mGovDebt_day = zeros(length(vDates_day),numel(aCountries));
for i = 1:iN
    mGovDebt_day(:,i)   = interp1(vDates_GovDebt, mGovDebt(:,i),vDates_day,'linear','extrap');
    aGovDebt{1,i}       = string(aCountries(i));
    aGovDebt{2,i}       = [vDates_day,mGovDebt_day(:,i)];
end
%save('aGovDebt', 'aGovDebt');

%Create market depth variable defined as gov debt - cumulative purchases.
%Save as .mat file.
aMktDepth = cell(2,numel(aCountries));
for i = 1:numel(aCountries)
    aMktDepth{1,i} = string(aCountries(i));
    aMktDepth{2,i} = [vDates_day,mGovDebt_day(:,i)-cumsum(aDH_interp_blackout_diff{2,i}(:,2))]; %Note that GovDebt is in millions and the purchases in billions. Transform everything into billions.
end
%save('aMktDepth', 'aMktDepth');

%Plot Data
mGovDebt  = mGovDebt(:,[1:3,5:7,9,11:12]);
mGovDebt_day= mGovDebt_day(:,[1:3,5:7,9,11:12]);
aMktDepth = aMktDepth(:,[1:3,5:7,9,11:12]);
aCountries = string(aMktDepth(1,:));
mMktDepth      =[];

vXticks         = datenum("01-01-2000",'dd-mm-yyyy');
vXticklabels    = "01-00";%"01-01-2000";
for i=2:2:17
    if i<10
        vXticks      = [vXticks;datenum("01-01-200"+i,'dd-mm-yyyy')]; %#ok<*AGROW>
        vXticklabels = [vXticklabels;"01-0"+i]; %[vXticklabels;"01-01-200"+i]; 
    else
        vXticks = [vXticks;datenum("01-01-20"+i,'dd-mm-yyyy')];
        vXticklabels = [vXticklabels;"01-"+i];%[vXticklabels;"01-01-20"+i];
    end        
end

figure
j=1;
for i=1:9
    subplot(5,2,i)
    hold on
    plot(vDates_day, mGovDebt_day(:,i),'LineStyle','-','Color','k')
    plot(vDates_day, aMktDepth{2,i}(:,2),'LineStyle','--','Color','k')
    hold off
    datetick('x' ,'mm-yy')
    xlim([aMktDepth{2,i}(1,1), aMktDepth{2,i}(end,1)])    
    xticks(vXticks)
    xticklabels(vXticklabels)
    ylabel('euro (billions)')
    title(aCountries(i))
    
end
subplot(5,2,10)
hold on
plot(0, 0,'LineStyle','-','Color','k')
plot(0, 0,'LineStyle','--','Color','k')
hold off
axis off
legend("Sovereign Debt", "Floating Supply", "location", "none")
