%Function SummaryStatistics.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load auction data from parent directory and
%         create Latex summary statistics tables.
%
%Inputs:    - aAUC.mat              Matlab-file with auction data.
%           - aYields_change.mat    Matlab-file with yield data.
%
%Outputs:   - Latex table code in the command window.
clc;clear;close all;

%------Load-Relevant-Data-------------------------------------------------%
load('../aDH_monthly.mat');
load('../aDH_interp_blackout_diff.mat')
load('../aMktDepth.mat')
aCountries               = string(aDH_interp_blackout_diff(1,[1:3,5:7,9,11:12])); %retain only the 9 complete countries.
aDH_monthly              = aDH_monthly(:,[1:3,5:7,9,11:12]);
aDH_interp_blackout_diff = aDH_interp_blackout_diff(:,[1:3,5:7,9,11:12]);
aMktDepth                = aMktDepth(:,[1:3,5:7,9,11:12]);

iN          = size(aCountries,2);
vDates      = aDH_interp_blackout_diff{2,1}(:,1);
iQE         = find(vDates>=datenum('10-05-2010','dd-mm-yyyy'),1,'first');
%-------------------------------------------------------------------------%


%------CB-Debt-Holdings-Figure--------------------------------------------%
%Extrapolate holdings data to obtain a entry for the first day of January
%1999. This observation is needed to obtain data for the first month.
vDebtSecHoldStart = zeros(1,numel(aCountries));
for i = 1:numel(aCountries)
    vDebtSecHoldStart(i) = interp1(aDH_monthly{2,i}(:,1), aDH_monthly{2,i}(:,2),vDates(1),'pchip','extrap');
end

%Create the interpolated Debt Holdings series.
aDH_interp = cell(2,numel(aCountries));
aDH_interp_diff = cell(2,numel(aCountries));
for i = 1:numel(aCountries)
    aDH_interp{1,i}    = aCountries(i);
    vDH_interp           = zeros(length(vDates),1);
    
    iLoc_1               = find(vDates>datenum('31-12-1998', 'dd-mm-yyyy'),1,'first');
    for t=1:length(aDH_monthly{2,1})
        if t~=1
            iLoc_1 = find(vDates>aDH_monthly{2,1}(t-1),1,'first');
        end
        iLoc_2 = find(vDates<=aDH_monthly{2,1}(t),1,'last');
        if t==1
            vDH_interp(iLoc_1:iLoc_2)      = (aDH_monthly{2,i}(t,2)-vDebtSecHoldStart(i))/(iLoc_2-iLoc_1+1).*(1:iLoc_2-iLoc_1+1)+vDebtSecHoldStart(i);
        else
            vDH_interp(iLoc_1:iLoc_2)      = (aDH_monthly{2,i}(t,2)-aDH_monthly{2,i}(t-1,2))/(iLoc_2-iLoc_1+1).*(1:iLoc_2-iLoc_1+1)+aDH_monthly{2,i}(t-1,2) ;
        end
    end
    %Transformed data to billions, the scale of the coefficient was so
    %small since it was in millions. makes more sense to look at billions.
    aDH_interp{2,i}    = [vDates,vDH_interp/1000];
    
end

%Plot Data
sLineStyle =  repmat(["-" ;"--" ;":"],[3 1]);
sMarker    =  [repmat("none",[3, 1]);...
              repmat("d",[3, 1]);...
              repmat(".",[3, 1])];
sColor     = [repmat([0  ,0  ,0  ],[3, 1]);...
              repmat([0.4,0.4,0.4],[3, 1]);...
              repmat([0.1,0.1,0.1],[3, 1])];
          
vXticks         = datenum("01-01-2000",'dd-mm-yyyy');
vXticklabels    = "01-00";%"01-01-2000";
for i=2:2:17
    if i<10
        vXticks      = [vXticks;datenum("01-01-200"+i,'dd-mm-yyyy')]; %#ok<*AGROW>
        vXticklabels = [vXticklabels;"01-0"+i]; %[vXticklabels;"01-01-200"+i]; 
    else
        vXticks = [vXticks;datenum("01-01-20"+i,'dd-mm-yyyy')];
        vXticklabels = [vXticklabels;"01-"+i];%[vXticklabels;"01-01-20"+i];
    end        
end
%
          
figure
hold on
for i =1:numel(aCountries)
    plot(aDH_monthly{2,i}(:,1),aDH_monthly{2,i}(:,2)/1000,'LineStyle',sLineStyle(i),...
          'Marker',sMarker(i),'Color',sColor(i,:), "MarkerSize", 3)
    datetick('x' ,'mm-yy')
    xlim([vDates(1), vDates(end)])
    xticks(vXticks)
    xticklabels(vXticklabels)
    
    ylabel('euro (billions)')
end
legend(aCountries, 'location', 'northwest')
%title('EMU Government Debt Holdings')
hold off
%-------------------------------------------------------------------------%


%------Debt-Purchases-Measure-Statistics----------------------------------%
fprintf('DH Stats\n')
B = [];

mDH = cell2mat(aDH_interp_blackout_diff(2,1:end));
mDH = mDH(:,(1:iN)*2); %exclude date stamps.

figure
%plot(vDates,max(mDH,0))
hold on
for i =1:numel(aCountries)
    plot(vDates,max(mDH(:,i),0))
end
hold off
datetick('x' ,'mm-yyyy')
xlim([vDates(1), vDates(end)])
ylabel('euro (billions)')
legend(aCountries, 'location', 'northwest')
title('max(\Delta EMU Government Debt Holdings,0)')


figure
sLineStyle =  "-";
sMarker    =  "none";
hold on
for i=1:iN
    subplot(5,2,i)
    plot(vDates,max(mDH(:,i),0),'LineStyle',sLineStyle,...
        'Marker',sMarker,'Color','k', "MarkerSize", 2)
    datetick('x' ,'mm-yyyy')
    xlim([vDates(1), vDates(end)])
    xticks(vXticks)
    xticklabels(vXticklabels)
    %ylim([0 0.9])
    ylabel('euro (billions)')
    title(aCountries(i))
end



mMeasure = max(mDH,0)*1000;
for i=1:iN
      B = [B;...
          [[mean(mMeasure(:,i)), mean(mMeasure(1:iQE-1,i)),mean(mMeasure(iQE:end,i))];...
           [std(mMeasure(:,i)), std(mMeasure(1:iQE-1,i)),std(mMeasure(iQE:end,i))]] ]; %#ok<AGROW>
end
T = array2table(B);
disp(T)
input.data = T; % use this table as input in input struct.
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input
fprintf('\n\n')
%-------------------------------------------------------------------------%


%------Floating-Supply-Measure-Statistics---------------------------------%
fprintf('Floating Supply Depth Stats\n')
B = [];
mMD = cell2mat(aMktDepth(2,1:end));
mMD = mMD(:,(1:iN)*2); %exclude date stamps.
mMeasure = mMD;
for i=1:iN
      B = [B;...
          [[mean(mMeasure(:,i)), mean(mMeasure(1:iQE-1,i)),mean(mMeasure(iQE:end,i))];...
           [std(mMeasure(:,i)), std(mMeasure(1:iQE-1,i)),std(mMeasure(iQE:end,i))]] ]; %#ok<AGROW>
end
T = array2table(B);
disp(T)
input.data = T; % use this table as input in input struct.
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input
fprintf('\n\n')
%-------------------------------------------------------------------------%


%------Scaled-Debt-Holdings-Measure-Statistics----------------------------%
fprintf('Scaled DH Stats\n')
B = [];
mMeasure = max(mDH,0)./(mMD/1000)*1000;
for i=1:iN
      B = [B;...
          [[mean(mMeasure(:,i)), mean(mMeasure(1:iQE-1,i)),mean(mMeasure(iQE:end,i))];...
           [std(mMeasure(:,i)), std(mMeasure(1:iQE-1,i)),std(mMeasure(iQE:end,i))]] ]; %#ok<AGROW>
end
T = array2table(B);
disp(T)
input.data = T; % use this table as input in input struct.
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input
%-------------------------------------------------------------------------%