%Function SummaryStatisticsVola.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load market volatility data from parent directory and
%         create Latex summary statistics tables.
%
%Inputs:    - aYields_Vola_Factor_individual.mat  Matlab-file with market volatility data.
%           - aDH_interp_blackout_diff.mat        Matlab-file with asset purchases data. (note that this data is needed for the interaction term)
%
%Outputs:   - Latex table code in the command window, figure containing the market volatility series for each country.
clc;clear;
load('../../Data/Market Volatility/Factor Analysis/aYields_Vola_Factor_individual.mat')
load('../aDH_interp_blackout_diff.mat')
load('../aMktDepth.mat')
aYields_Vola_Factor_individual= aYields_Vola_Factor_individual([1,3],[1:3,5:7,9,11,12]);

aCountries  = string(aYields_Vola_Factor_individual(1,:));
iN          = size(aYields_Vola_Factor_individual,2); %number of countries.

%Dates
vDates = aYields_Vola_Factor_individual{2,1}(:,1);
iLoc_QE = find(vDates == datenum('10-05-2010', 'dd-mm-yyyy')); %Start QE.

%Only 10-year market
mVola = cell2mat(aYields_Vola_Factor_individual(2,:));
mVola = mVola(:,(1:iN)*2);
clear aYields_Vola_Factor_SqDevFromMean

%Delete greece, ireland, luxembourg
mDH = cell2mat(aDH_interp_blackout_diff(2,[1:3,5:7,9,11,12]));
mMD = cell2mat(aMktDepth(2,[1:3,5:7,9,11:12]));
mDH = mDH(1:end-3,(1:iN)*2); %get rid of date stamps.
mMD = mMD(1:end-3,(1:iN)*2); %exclude date stamps.
%clear aDH_interp_diff aMktDepth

%Interaction between market volatility and Delta DH.
mVolDH = mVola.*mDH./(mMD/1000);

%Create Latex Table
mTable = [];
for i=1:9
    mTable = [mTable;[ [mean(mVola(:,i)),mean(mVola(1:iLoc_QE-1,i)),mean(mVola(iLoc_QE:end,i))];...
                       [std(mVola(:,i)) ,std(mVola(1:iLoc_QE-1,i)) ,std(mVola(iLoc_QE:end,i))] ] ]; %#ok<AGROW>
end

for i=1:size(mDH,2) %now add stats of the interaction with Delta DH.
    mTable = [mTable;[ [mean(mVolDH(:,i)),mean(mVolDH(1:iLoc_QE-1,i)),mean(mVolDH(iLoc_QE:end,i))];...
                       [std(mVolDH(:,i)) ,std(mVolDH(1:iLoc_QE-1,i)) ,std(mVolDH(iLoc_QE:end,i))] ] ];
end
T = array2table(mTable);
disp(T)
input.data = T; % use this table as input in input struct.
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input

%plot
vXticks         = datenum("01-01-2000",'dd-mm-yyyy');
vXticklabels    = "01-00";%"01-01-2000";
for i=2:2:17
    if i<10
        vXticks      = [vXticks;datenum("01-01-200"+i,'dd-mm-yyyy')]; %#ok<*AGROW>
        vXticklabels = [vXticklabels;"01-0"+i]; %[vXticklabels;"01-01-200"+i]; 
    else
        vXticks = [vXticks;datenum("01-01-20"+i,'dd-mm-yyyy')];
        vXticklabels = [vXticklabels;"01-"+i];%[vXticklabels;"01-01-20"+i];
    end        
end
figure
for i=1:iN
    subplot(5,2,i)
    plot(vDates,mVola(:,i),'-k')
    datetick('x','mm-yy')
    xlim([vDates(1),vDates(end)])
    xticks(vXticks)
    xticklabels(vXticklabels)
    title(aCountries(i))
end