%Function SummaryStatistics.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load auction data from parent directory and
%         create Latex summary statistics tables.
%
%Inputs:    - aAUC.mat              Matlab-file with auction data.
%           - aYields_change.mat    Matlab-file with yield data.
%
%Outputs:   - Latex table code in the command window.
clc;clear;
load('../../Data/Yields/aYields_change.mat')
load('../aAUC.mat')

aCountries = aAUC(1,:);

%Specify which countries to include in the regression per maturity.
[~,aCountrySet,~]    = xlsread("../../Data/CountrySetPerMaturity.xlsx","B1:Z3");
aSetCntry            = cell(3,1);
[~,aSetCntry{1,1},~] = intersect(aCountries,aCountrySet(1,:));   %5-Year maturity.
[~,aSetCntry{2,1},~] = intersect(aCountries,aCountrySet(2,:));   %10-Year maturity.
[~,aSetCntry{3,1},~] = intersect(aCountries,aCountrySet(3,:));   %30-Year maturity.

%------Yield-Changes------------------------------------------------------%
% 10Year only
iN              = numel(aSetCntry{2,1});
mYields_change  = cell2mat(aYields_change(3,aSetCntry{2,1}));
mYields_change  = mYields_change(:,(1:iN)*2)*100; %Get rid of the day vectors.

iLoc_APP        = find(aYields_change{3,1}(:,1)>= datenum('10-05-2010'),1,'first');
fprintf('Summary Statistics\n\n')
fprintf('Yield Changes in bps\n')
T1 = array2table( [[mean(mYields_change,1)', mean(mYields_change(1:iLoc_APP-1,:),1)',...
     mean(mYields_change(iLoc_APP:end,:),1)'];[std(mYields_change,1)', std(mYields_change(1:iLoc_APP-1,:),1)',...
     std(mYields_change(iLoc_APP:end,:),1)']],'VariableNames',{'FullSample', 'PreAPP', 'APP'},...
     'Rownames',cellstr([aCountries(1,aSetCntry{2,1}),"stdev-"+aCountries(1,aSetCntry{2,1})]));
disp(T1);
input.data = T1; % use this table as input in input struct.
%input.tableColLabels = {'','0.2','0.4','0.6','0.8','\sigma_{0.2}','\sigma_{0.4}','\sigma_{0.6}','\sigma_{0.8}'};
%input.tableRowLabels = {'0.0','0.2','0.4','0.6', '0.8', '1.0','0.0','0.2','0.4','0.6', '0.8', '1.0','0.0','0.2','0.4','0.6', '0.8', '1.0','0.0','0.2','0.4','0.6', '0.8', '1.0'};
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input

%mCorr  = corr(mYields_change);
%mCorr  = corr(mYields_change(1:iLoc_APP-1,:));
mCorr  = corr(mYields_change(iLoc_APP:end,:));
T1 = array2table( mCorr ,'VariableNames',aCountries(1,aSetCntry{2,1}),...
     'Rownames',aCountries(1,aSetCntry{2,1}) );
disp(T1);
input.data = T1; % use this table as input in input struct.
%input.tableColLabels = {'','0.2','0.4','0.6','0.8','\sigma_{0.2}','\sigma_{0.4}','\sigma_{0.6}','\sigma_{0.8}'};
%input.tableRowLabels = {'0.0','0.2','0.4','0.6', '0.8', '1.0','0.0','0.2','0.4','0.6', '0.8', '1.0','0.0','0.2','0.4','0.6', '0.8', '1.0','0.0','0.2','0.4','0.6', '0.8', '1.0'};
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input
corr(mYields_change)
fprintf('\n\n')
%-------------------------------------------------------------------------%


%------Auction-Statistics-------------------------------------------------%
fprintf('Auctions Stats\n')
%load('../aAUC.mat')
load('../aAC.mat')
load('../aAY.mat')
B   = [];
aAUC= aAUC([1,3],aSetCntry{2,1}); %retain the 10-year maturity bucket for the countries of interest.
aAC = aAC([1,3],aSetCntry{2,1});
aAY = aAY([1,3],aSetCntry{2,1});

vDates=aAUC{2,1}(:,1);
iQE   = find(vDates>=datenum('10-05-2010','dd-mm-yyyy'),1,'first');

for i=1:iN
      B = [B;...
          [[sum(aAUC{2,i}(:,2)), sum(aAUC{2,i}(1:iQE-1,2)),sum(aAUC{2,i}(iQE:end,2))];...
          [mean(nonzeros(aAC{2,i}(:,2)), 'omitnan'), mean(nonzeros(aAC{2,i}(1:iQE-1,2)), 'omitnan'),mean(nonzeros(aAC{2,i}(iQE:end,2)), 'omitnan')];...
          [mean(nonzeros(aAY{2,i}(:,2)), 'omitnan'), mean(nonzeros(aAY{2,i}(1:iQE-1,2)), 'omitnan'),mean(nonzeros(aAY{2,i}(iQE:end,2)), 'omitnan')]] ]; %#ok<AGROW>
            
       if i==8
           disp(mean(nonzeros(aAY{2,i}(:,2))))
       end
end
T = array2table(B);
disp(T)
input.data = T; % use this table as input in input struct.
input.dataFormat = {'%.2f'}; % Set the row format of the data values.
input.tableColumnAlignment = 'r';% Column alignment.
input.tableBorders = 0;% Switch table borders on/off.
input.makeCompleteLatexDocument = 0;% Switch to generate a complete LaTex document or just a table.
input.tableCaption = ' ';
input.tableLabel = 'tab:';
input.tablePositioning = 'h';
latex = latexTable(input);% Now call the function to generate LaTex code.
clear T1 input
fprintf('\n\n')
%-------------------------------------------------------------------------%



load('../../Data/Yields/aYields.mat')
vDates  = aYields{3,1}(:,1);
aYields = aYields(:,[1:3,5:7,9,11:12]);
mYields = cell2mat(aYields(2,:));
mYields = mYields(:,2:2:size(mYields,2));
iN = size(mYields,2);
plot(repmat(vDates,[1 iN]), mYields)
datetick('x','mm-yy');
legend(string(aYields(1,:)))