%Function GovDebt_vs_CreditRating.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Load credit rating data, and compare credit ratings over the sample period to assess robustness of our clusters.
%
%Inputs:    - 
%
%Outputs:   - Figures depicting evolution of credit rating data of subject countries.


clear;close all; clc;
%Read data from files.

cRating = "D";
% iRating = k;
% S&P:      k=B
% Moody's:  k=C
% Fitch:    k=D
% DBRS:     k=E

mFitchSPConversion = ["D","D";"SD","RD";repmat({"C";"CC";"CCC-";"CCC";...
                       "CCC+";"B-";"B";"B+";"BB-";"BB";"BB+";"BBB-";...
                       "BBB";"BBB+";"A-";"A";"A+";"AA-";"AA";"AA+";...
                       "AAA"},[1,2])];

vCountries = ["Austria";"Belgium";"Germany";"Spain";"Finland";"France";...
              "Italy";"Netherlands";"Portugal"];                              %AT, BE, DE, ES, FI, FR, IT, NL, PT (ECB standards)
iN         = numel(vCountries);

%Read dates vector from "Auctions.xlsx".
vDates      = x2mdate(xlsread("../Data/Auctions/Auctions.xlsx","A:A"));
iT          = length(vDates);


%Load Gov Debt (yearly) stats---------------------------------------------%
vDates_debt = x2mdate(xlsread("../Data/Government Debt/Debt_to_GDP","A:A"),1);
mDebt       = xlsread("../Data/Government Debt/Debt_to_GDP","B:J");

mDebt       = mDebt(find(vDates_debt>vDates(1),1,'first'):end,:);
vDates_debt = vDates_debt(find(vDates_debt>vDates(1),1,'first'):end);       %Retain time period of our sample

%Plot
figure
plot(repmat(vDates_debt,[1,iN]),mDebt)
datetick('x','mm-yy')
xlim([vDates_debt(1),vDates_debt(end)])
ylabel("Debt to GDP (%)")
%ylim([12 23])
legend(vCountries,'location','southwest')
%-------------------------------------------------------------------------%

%Load Credit Rating stats-------------------------------------------------%
aRating     = string(zeros(iT,iN));
mRating     = zeros(iT,iN);



for i=1:numel(vCountries)
    vDates_i       = x2mdate(xlsread("../Data/Credit Ratings/"+vCountries(i)+...
                                    "_credit_rating.xlsx","A:A"));
    %[~,aRating_i] = xlsread("Credit Ratings/"+vCountries(i)+...
    %                                "_credit_rating.xlsx",cRating+2":"+cRating+(length(vDates_i)+1));
    [~,~,aRating_Fitch_i] = xlsread("../Data/Credit Ratings/"+vCountries(i)+...
                                "_credit_rating.xlsx","D"+2+":D"+(length(vDates_i)+1));
    aRating_Fitch_i = string(aRating_Fitch_i);                
                        
    %[~,~,aRating_SP_i]    = xlsread("Credit Ratings/"+vCountries(i)+...
    %                            "_credit_rating.xlsx","B"+2+":B"+(length(vDates_i)+1));   
    %aRating_SP_i = string(aRating_SP_i); 
    
    aRating_i       = string(zeros(length(vDates_i),1))+NaN;                    %aRating_Fitch_i+aRating_SP_i; doesnt work (unequal length).
    aRating_i(~ismissing(aRating_Fitch_i)) = aRating_Fitch_i(~ismissing(aRating_Fitch_i));
    %aRating_i(~ismissing(aRating_SP_i))    = aRating_SP_i(~ismissing(aRating_SP_i));
    
    vNonEmpty      = ~ismissing(aRating_i); %Retain rating moments of specific agency only.
    vDates_i       = vDates_i(vNonEmpty);
    aRating_i      = aRating_i(vNonEmpty);
    
    aRating_i(strcmp(aRating_i,"RD")) = "SD"; %convert any RDs to SD for ease.
    [~,vRating_i]  = ismember(aRating_i,mFitchSPConversion(:,1));             %covert Fitch rating to number (ranking)
    
    [~,vIndex_t,vIndex_k]    = intersect(vDates,vDates_i);
    
    iLoc_firstRating = find(vDates_i<=vDates(1),1,'last');
    iLoc_lastRating  = find(vDates_i<=vDates(end),1,'last');
    vDates_i         = vDates_i(iLoc_firstRating:iLoc_lastRating); %Remove unnecessary data.
    vRating_i        = vRating_i(iLoc_firstRating:iLoc_lastRating);
    
    iLoc_prev = 0;
    if length(vDates_i)>=2
        for t=2:length(vDates_i)
            iLoc = find(vDates>=vDates_i(t),1,'first');
            aRating(iLoc_prev+1:iLoc,i) = aRating_i(t-1);
            mRating(iLoc_prev+1:iLoc,i) = vRating_i(t-1);
            iLoc_prev                   = iLoc;
        end
    else
        t =1; %Germany has only one rating moment. So skips the loop and fills in rating 23 for whole sample.
    end
    aRating(iLoc_prev+1:end,i) = aRating_i(t);
    mRating(iLoc_prev+1:end,i) = vRating_i(t);
    %if~isempty(vIndex_t)
    %    aRating(1:vIndex_t(1)-1,i) = aRating_i(iLoc_firstRating);
    %    aRating(1:vIndex_t(1)-1,i) = vRating_i(iLoc_firstRating);
    %    for k=2:length(vIndex_t)
    %        aRating(vIndex_t(k-1):vIndex_t(k)-1,i) = aRating_i(vIndex_k(k)-1);
    %        mRating(vIndex_t(k-1):vIndex_t(k)-1,i) = vRating_i(vIndex_k(k)-1);
    %    end
    %else
    %    aRating(:,i) = aRating_i(iLoc_firstRating);
    %    mRating(:,i) = vRating_i(iLoc_firstRating);
    %end
    
    %if i==1
    %    disp(datestr(vDates_i(iLoc_firstRating)))
    %    disp(' ')
    %    disp(datestr(vDates_i))
    %    cc
    %end
end

%Plot
figure
plot(repmat(vDates,[1,iN]),mRating)
datetick('x','mm-yy')
xlim([vDates(1),vDates(end)])
yticks((1:length(mFitchSPConversion(:,1))))
yticklabels(cellstr(mFitchSPConversion(:,1)))
%set(gca,'yticklabel',(mFitchSPConversion(:,1)))
ylim([12 23])
legend(vCountries,'location','southwest')
%-------------------------------------------------------------------------%

%Debt vs Credit Rating----------------------------------------------------%
iDatePSPP  = datenum('10-05-2010','dd-mm-yyyy');%Last day before sov debt purchases
%iDatePSPP  = datenum('31-12-2009','dd-mm-yyyy');

%Pre APP
vCrR_av_pre     = mean(mRating(1:find(vDates<=iDatePSPP,1,'last'),:),1);
vDebt_av_pre    = mean(mDebt(1:find(vDates_debt<=iDatePSPP,1,'last'),:),1);
%Post APP
vCrR_av_post    = mean(mRating(find(vDates>iDatePSPP,1,'first'):end,:),1);
vDebt_av_post   = mean(mDebt(find(vDates_debt<=iDatePSPP,1,'last')+1:end,:),1); %last + 1 to include the year 2015 (PSPP launched in march.

%Plot
figure
subplot(3,1,1)
gscatter(mean(mRating,1),mean(mDebt,1),vCountries)
xticks((1:length(mFitchSPConversion(:,1))))
xticklabels(cellstr(mFitchSPConversion(:,1)))
xlim([12 23.5])
ylim([40 140])
xlabel("Credit Rating")
ylabel("Debt to GDP (%)")
legend(vCountries','location','southwest')
title("Whole Sample")

subplot(3,1,2)
gscatter(vCrR_av_pre,vDebt_av_pre,vCountries)
xticks((1:length(mFitchSPConversion(:,1))))
xticklabels(cellstr(mFitchSPConversion(:,1)))
xlim([12 23.5])
ylim([40 140])
xlabel("Credit Rating")
ylabel("Debt to GDP (%)")
legend(vCountries','location','southwest')
title("Pre APP")
%title("Pre Sovereign Debt Crisis")

subplot(3,1,3)
gscatter(vCrR_av_post,vDebt_av_post,vCountries)
xticks((1:length(mFitchSPConversion(:,1))))
xticklabels(cellstr(mFitchSPConversion(:,1)))
xlim([12 23.5])
ylim([40 140])
xlabel("Credit Rating")
ylabel("Debt to GDP (%)")
legend(vCountries','location','southwest')
title("Post APP")
%title("Post Sovereign Debt Crisis")


figure
gscatter(vCrR_av_post,vDebt_av_post,vCountries)
xticks((1:length(mFitchSPConversion(:,1))))
xticklabels(cellstr(mFitchSPConversion(:,1)))
xlim([12 23.5])
ylim([40 140])
xlabel("Credit Rating")
ylabel("Debt to GDP (%)")
legend(vCountries','location','southwest')
%-------------------------------------------------------------------------%