clear;clc;close all;
load("../../../Data/Yields/aYields_change.mat")
load("../../../Data/Market Volatility/Factor Analysis/aYields_Vola_Factor_individual.mat")
load("../../aAUC.mat")
load("../../aDH_interp_blackout_diff.mat")
load("../../aMktDepth.mat")
load("../../../Data/Market Conditions/aFinMarket.mat")

aYields_Vola = aYields_Vola_Factor_individual;
clear aYields_Vola_Factor_individual;

aMaturities = ["5Y";"10Y";"30Y"];
aCountries  = aAUC(1,:);
iN          = numel(aCountries);
vDates      = aAUC{2,1}(:,1);
iDaysBlckOut= 2; %pre-auction day and auction day.
bCovariates = 1;

%Specify which countries to include in the regression per maturity.
[~,aCountrySet,~]    = xlsread("../../../Data/CountrySetPerMaturity.xlsx","B1:Z3");
aSetCntry            = cell(3,1);
[~,aSetCntry{1,1},~] = intersect(aCountries,aCountrySet(1,:));   %5-Year maturity.
[~,aSetCntry{2,1},~] = intersect(aCountries,aCountrySet(2,:));   %10-Year maturity.
[~,aSetCntry{3,1},~] = intersect(aCountries,aCountrySet(3,:));   %30-Year maturity.

%Specify country clusters
[~,~,aCluster]    = xlsread("../../../Data/Clusters.xlsx","B1:M2");
vClusters = cell2mat(aCluster(2,:)); %vClusters = vClusters(~isnan(vClusters));
iClusters = length(unique(vClusters));

%Specify locations in the results excel-file for the estimate. Note that
%beta locations need to be specified per cluster.
aLoc = ["2";"8";"3";"9";"4";"10";"5";"11";"6";"12";...    %domestic effects
        "14";"26";"15";"27";"16";"28";"17";"29";"18";"30";...%foreign effects from cluster 1 
        "20";"32";"21";"33";"22";"34";"23";"35";"24";"36"];  %foreign effects from cluster 2
iK   = length(aLoc);

%Specify time window around auction. iCentre specifies the location of the
%peak of the auction cycle (0 is auction day itself, positive numbers are days
%after the auction and negative are pre-auction.
%iDays_Pre and iDays_Post specify the time window before and after iCentre.
iCentre    = 0;
iDays_Pre  = 5;
iDays_Post = 5;
iWindow_len= iDays_Pre+iDays_Post;

% Include stuff for writing to Excel file (if using Mac), otherwise,
% xlwrite should be changed to xlswrite.
javaaddpath('../../xlwrite/poi_library/poi-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/poi-ooxml-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/poi-ooxml-schemas-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/xmlbeans-2.3.0.jar');
javaaddpath('../../xlwrite/poi_library/dom4j-1.6.1.jar');
javaaddpath('../../xlwrite/poi_library/stax-api-1.0.1.jar');

aOutput      = cell(2,1);
aOutput{1,1} = "B";

iLoc_QE     = find(vDates == datenum('10-05-2010', 'dd-mm-yyyy')); %Start SMP.
vDates_QE   = vDates(iLoc_QE:end-3); %yield data only goes until 27th dec. skip last 3.

if bCovariates
    %Differences for EONIA, Log Differences for Indices.
    mX_covs = [aFinMarket{2,1}(iLoc_QE:end-3,2),log(aFinMarket{2,2}(iLoc_QE:end-3,2)),...
        log(aFinMarket{2,3}(iLoc_QE:end-3,2)),log(aFinMarket{2,4}(iLoc_QE:end-3,2))];
    %mX_covs = [mX_covs(1,:);diff(mX_covs)];
    mX_covs = diff(mX_covs);
    mX_covs(:,2:4) = mX_covs(:,2:4)*100;
    mX_covs = [ones(2,4)*NaN;mX_covs(1:end-1,:)]; %first two lagged differences we don't observe; these observations are filtered out as we drop observations due to auction dummies.
end

for m=2:2
    disp('maturity')
    disp(m)
    iN = numel(aSetCntry{m,1});
    
    aYields_change_m    = aYields_change(:,aSetCntry{m,1});
    aAUC_m              = aAUC(:,aSetCntry{m,1});
    aYields_Vola_m      = aYields_Vola(:,aSetCntry{m,1});
    aDH_interp_diff_m   = aDH_interp_blackout_diff(:,aSetCntry{m,1});
    vClusters_m         = vClusters(1,aSetCntry{m,1});
    aMktDepth_m         = aMktDepth(:,aSetCntry{m,1});
    iN_c                = numel(aYields_change_m(1,:));
    
    %REALLOCATE FRANCE TO CLUSTER 1.
    vClusters_m(6) = 1; %Only France reallocation
    
    %------Dependent-Variable-----------------------------------------%
    vY_all =[];
    vID    =[];
    for i=1:iN_c
        vY_diff = aYields_change_m{m+1,i}(iLoc_QE:end,2)*100;                 %in bps
        iT_total= length(vY_diff);
        vY_all  = [vY_all,vY_diff];                             %#ok<AGROW>
        vID     = [vID,ones(iT_total,1)*i];                     %#ok<AGROW>
    end
    %-----------------------------------------------------------------%
    
    %------Independent-Variables--------------------------------------%
    iLag        = 21;
    mAUC        = zeros(iT_total,iN_c);
    mVola       = zeros(iT_total,iN_c);
    mDH         = zeros(iT_total,iN_c);
    mMktDepth   = zeros(iT_total,iN_c);
    for i=1:iN_c
        mAUC(:,i)       = aAUC_m{m+1,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2);
        mVola(:,i)      = aYields_Vola_m{m+1,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2);
        mDH(:,i)        = max(aDH_interp_diff_m{2,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2),0);%aDH_interp_diff_m{2,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2);%max(aDH_interp_diff_m{2,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2),0);
        mMktDepth(:,i)  = aMktDepth_m{2,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2)/1000;
        %mMktDepth(:,i)  = aMktDepth_m{2,i}( iLoc_QE-iLag:iLoc_QE+iT_total-1-iLag ,2)/1000;
    end
    %mAUC_total    = sum(mAUC,2); %total auctions at time t, easy to create the foreign dummy.
    mAUC_total_C1 = sum(mAUC(:,vClusters_m == 1),2); %summed over the countries in cluster 1
    mAUC_total_C2 = sum(mAUC(:,vClusters_m == 2),2); %summed over the countries in cluster 2
    %-----------------------------------------------------------------%
    
    %------Panel-Regression-------------------------------------------%
    mX =[];
    for i=1:iN_c
        mX_i            = zeros(iT_total,iN_c+iK*iWindow_len+4*iN_c);
        vIntercept      = zeros(iN_c,1);
        vIntercept(i)   = 1;
        for t=1+iDays_Post:iT_total-(iDays_Pre-1)
            vRange          = t+(iDays_Pre-1):-1:t-iDays_Post;
            vX_dom          = mAUC(vRange,i);
            vX_dom_VOL      = vX_dom.*mVola(t,i);
            vX_dom_DH       = vX_dom.*mDH(t,i);
            vX_dom_VOL_DH   = vX_dom.*mVola(t,i).*mDH(t,i);
            vX_dom_mkt      = vX_dom.* mMktDepth(t,i);
            
            %vX_for       = (mAUC_total(vRange)-vX_dom);
            if vClusters_m(i) == 1
                vX_for_C1_i = (mAUC_total_C1(vRange)-vX_dom);
                vX_for_C2_i = (mAUC_total_C2(vRange));
            else %elseif vClusters_m(i) == 2
                vX_for_C1_i = (mAUC_total_C1(vRange));
                vX_for_C2_i = (mAUC_total_C2(vRange)-vX_dom);
            end
            
            %vX_for_C1_vol_i= vX_for_C1_i.*mVola(t,i);
            %vX_for_C2_vol_i= vX_for_C2_i.*mVola(t,i);
            vX_for_C1_vol_i= sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                mVola(t,setdiff(find(vClusters_m==1),i)),2);
            vX_for_C2_vol_i= sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                mVola(t,setdiff(find(vClusters_m==2),i)),2);
            
            %vX_for_DH    = sum(mAUC(vRange,1:end~=i).*mDH(t,1:end~=i),2);
            vX_for_C1_DH    = sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                mDH(t,setdiff(find(vClusters_m==1),i)),2);
            vX_for_C2_DH    = sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                mDH(t,setdiff(find(vClusters_m==2),i)),2);
            
            %vX_for_VOL_DH= sum(mAUC(vRange,1:end~=i).*mVola(t,i).*mDH(t,1:end~=i),2);
            vX_for_C1_VOL_DH = sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                mVola(t,setdiff(find(vClusters_m==1),i)).*...
                mDH(t,setdiff(find(vClusters_m==1),i)),2);
            vX_for_C2_VOL_DH = sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                mVola(t,setdiff(find(vClusters_m==2),i)).*...
                mDH(t,setdiff(find(vClusters_m==2),i)),2);
            
            %MktDepth effect
            vX_for_C1_mkt    = sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                mMktDepth(t,setdiff(find(vClusters_m==1),i) ),2);
            vX_for_C2_mkt    = sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                mMktDepth(t,setdiff(find(vClusters_m==2),i) ),2);
            
            mX_i(t,:)   = [vIntercept',...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_VOL',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_VOL_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_mkt',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_vol_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_VOL_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_mkt',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_vol_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_VOL_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_mkt',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                mVola(t,:),mDH(t,:),mVola(t,:).*mDH(t,:), mMktDepth(t,:)];
        end
        if ~bCovariates
            mX = [mX;mX_i];                                 %#ok<AGROW>
        else
            mX = [mX;[mX_i,mX_covs]];                       %#ok<AGROW>
        end
    end
    %Only retain indices that contain data, i.e. cut out zeros.
    vIndex_i = [zeros(iDays_Post,1);ones(iT_total-iDays_Pre-iDays_Post+1,1);zeros((iDays_Pre-1),1)];
    vIndex   = repmat(vIndex_i , [iN_c 1]);
    
    vY_all = vY_all(find(vIndex));                             %#ok<*FNDSB>
    vID    = vID(find(vIndex));
    %mX     = mX(find(vIndex),:);
    mX     = mX(find(vIndex),setdiff((1:size(mX,2)),iN_c+[44,45,54,55,64,65,74,75,144,145,154,155,164,165,174,175,...
        244,245,254,255,264,265,274,275])); %Get rid of AUC_t-1*DH_t-1, AUC_t, DH_t interactions (orthogonal)
    iT     = sum(vIndex_i);
    %-----------------------------------------------------------------%
    
    %------Panel-Results----------------------------------------------%
    vBeta  = lsqlin(mX, vY_all,[],[], [],[],[],[]);%lsqlin(mX, vY_all,[],[], mConstr, zeros(iK*(iWindow_len-1),1),[],[]);
    vResid = vY_all-mX*vBeta;
    
    dSSR    = vResid'*vResid;
    dSST    = (vY_all-mean(vY_all))'*(vY_all-mean(vY_all));
    dR2_adj = R2_adjusted( dSSR, dSST, iT*iN_c, size(mX,2) );
    
    %Born Breitung (2016)
    [dBB_Stat, dBB_PVal] = Autocorrelation_BornBreitung(iN_c,iT,vResid);
    
    %C.B. Hansen (2007)
    [mCov, ~, ~] = SandwichCovariance_Arellano(size(mX,2),iN_c,iT,vResid,mX); %size(mX,2) = iWindow_len*ik+iN
    %-----------------------------------------------------------------%
    
    %------Write-Results----------------------------------------------%
    mLoc1 =[iN_c+1,iN_c+1+(iDays_Pre-1)+iCentre;... %10 DOM
        iN_c+11,iN_c+11+(iDays_Pre-1)+iCentre;... %10
        iN_c+21,iN_c+21+(iDays_Pre-1)+iCentre;... %10
        iN_c+31,iN_c+31+(iDays_Pre-1)+iCentre;... %10
        iN_c+41,iN_c+41+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+49,iN_c+49+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+57,iN_c+57+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+65,iN_c+65+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+73,iN_c+73+(iDays_Pre-1)+iCentre;... %10
        iN_c+83,iN_c+83+(iDays_Pre-1)+iCentre;... %10
        iN_c+93,iN_c+93+(iDays_Pre-1)+iCentre;... %10 FOR_C1
        iN_c+103,iN_c+103+(iDays_Pre-1)+iCentre;... %10
        iN_c+113,iN_c+113+(iDays_Pre-1)+iCentre;... %10
        iN_c+123,iN_c+123+(iDays_Pre-1)+iCentre;... %10
        iN_c+133,iN_c+133+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+141,iN_c+141+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
        iN_c+149,iN_c+149+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+157,iN_c+157+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
        iN_c+165,iN_c+165+(iDays_Pre-1)+iCentre;... %10
        iN_c+175,iN_c+175+(iDays_Pre-1)+iCentre;... %10
        iN_c+185,iN_c+185+(iDays_Pre-1)+iCentre;... %10   FOR_C2
        iN_c+195,iN_c+195+(iDays_Pre-1)+iCentre;... %10
        iN_c+205,iN_c+205+(iDays_Pre-1)+iCentre;... %10
        iN_c+215,iN_c+215+(iDays_Pre-1)+iCentre;... %10
        iN_c+225,iN_c+225+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+233,iN_c+233+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
        iN_c+241,iN_c+241+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
        iN_c+249,iN_c+249+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
        iN_c+257,iN_c+257+(iDays_Pre-1)+iCentre;... %10
        iN_c+267,iN_c+267+(iDays_Pre-1)+iCentre]; %10
    
    mLoc2 = [iN_c+1+(iDays_Pre-1)+iCentre+1,iN_c+10;...%10 DOM
        iN_c+11+(iDays_Pre-1)+iCentre+1,iN_c+20;...%10
        iN_c+21+(iDays_Pre-1)+iCentre+1,iN_c+30;...%10
        iN_c+31+(iDays_Pre-1)+iCentre+1,iN_c+40;...%10
        iN_c+41+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+48;...%8
        iN_c+49+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+56;...%8
        iN_c+57+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+64;...%8
        iN_c+65+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+72;...%8
        iN_c+73+(iDays_Pre-1)+iCentre+1,iN_c+82;...%10
        iN_c+83+(iDays_Pre-1)+iCentre+1,iN_c+92;...%10
        iN_c+93+(iDays_Pre-1)+iCentre+1,iN_c+102;...%10 FOR_C1
        iN_c+103+(iDays_Pre-1)+iCentre+1,iN_c+112;...%10
        iN_c+113+(iDays_Pre-1)+iCentre+1,iN_c+122;...%10
        iN_c+123+(iDays_Pre-1)+iCentre+1,iN_c+132;...%10
        iN_c+133+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+140;...%8
        iN_c+141+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+148;...%8
        iN_c+149+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+156;...%8
        iN_c+157+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+164;...%8
        iN_c+165+(iDays_Pre-1)+iCentre+1,iN_c+174;...%10
        iN_c+175+(iDays_Pre-1)+iCentre+1,iN_c+184;...%10
        iN_c+185+(iDays_Pre-1)+iCentre+1,iN_c+194;...%10 FOR_C2
        iN_c+195+(iDays_Pre-1)+iCentre+1,iN_c+204;...%10
        iN_c+205+(iDays_Pre-1)+iCentre+1,iN_c+214;...%10
        iN_c+215+(iDays_Pre-1)+iCentre+1,iN_c+224;...%10
        iN_c+225+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+232;...%8
        iN_c+233+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+240;...%8
        iN_c+241+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+248;...%8
        iN_c+249+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN_c+256;...%8
        iN_c+257+(iDays_Pre-1)+iCentre+1,iN_c+266;...%10
        iN_c+267+(iDays_Pre-1)+iCentre+1,iN_c+276];%10
    for k = 1:iK
        dEst= sum(vBeta(mLoc1(k,1):mLoc1(k,2))) - ...
            sum(vBeta(mLoc2(k,1):mLoc2(k,2))) ;
        
        mR = zeros(1,length(vBeta));
        vR = 0;
        mR(mLoc1(k,1):mLoc1(k,2))   = ones(1,mLoc1(k,2)-mLoc1(k,1)+1);
        mR(mLoc2(k,1):mLoc2(k,2))   = -ones(1,mLoc2(k,2)-mLoc2(k,1)+1);
        
        [dF,dPVal] = FtestHansen(vBeta,mCov,mR,vR,iN_c,length(vR));
        sLoc       = aLoc(k);
        xlwrite("ReallocationRobustness_results.xlsx", Write_significance(dEst, dPVal),...
            aMaturities(m),aOutput{1,1}+sLoc);
    end
    if bCovariates
        for i=1:4
            dEst        = vBeta(end-4+i);
            mR          = zeros(1,length(vBeta));
            vR          = 0;
            mR(end-4+i) = 1;
            [dF,dPVal]  = FtestHansen(vBeta,mCov,mR,vR,iN_c,length(vR));
            xlwrite("ReallocationRobustness_results.xlsx", Write_significance(dEst, dPVal),...
                aMaturities(m),aOutput{1,1}+(i+43));
        end
    end
    
    sLoc = "43";
    xlwrite("ReallocationRobustness_results.xlsx", Write_significance(dBB_Stat, dBB_PVal),aMaturities(m),aOutput{1,1}+sLoc);
    sLoc = "44";
    xlwrite("ReallocationRobustness_results.xlsx", dR2_adj,aMaturities(m),aOutput{1,1}+sLoc);
    sLoc = "45";
    xlwrite("ReallocationRobustness_results.xlsx", iT,aMaturities(m),aOutput{1,1}+sLoc);
    sLoc = "46";
    xlwrite("ReallocationRobustness_results.xlsx", iN_c,aMaturities(m),aOutput{1,1}+sLoc);
    %-----------------------------------------------------------------%
end