clear;clc;close all;
load("../../../Data/Yields/aYields_change.mat")
load("../../../Data/Market Volatility/Factor Analysis/aYields_Vola_Factor_individual.mat")
load("../../aAUC.mat")
load("../../aDH_interp_blackout_diff_hat.mat")
load("../../aGovDebt.mat")
load("../../../Data/Market Conditions/aFinMarket.mat")

aYields_Vola = aYields_Vola_Factor_individual;
clear aYields_Vola_Factor_individual;

aMaturities = ["5Y";"10Y";"30Y"];
aCountries  = aAUC(1,:);
iN          = numel(aCountries);
vDates      = aAUC{2,1}(:,1);
iDaysBlckOut= 2;

%Specify which countries to include in the regression per maturity.
[~,aCountrySet,~]    = xlsread("../../../Data/CountrySetPerMaturity.xlsx","B1:Z3");
aSetCntry            = cell(3,1);
[~,aSetCntry{1,1},~] = intersect(aCountries,aCountrySet(1,:));   %5-Year maturity.
[~,aSetCntry{2,1},~] = intersect(aCountries,aCountrySet(2,:));   %10-Year maturity.
[~,aSetCntry{3,1},~] = intersect(aCountries,aCountrySet(3,:));   %30-Year maturity.

%Specify country clusters
[~,~,aCluster]    = xlsread("../../../Data/Clusters.xlsx","B1:M2");
vClusters = cell2mat(aCluster(2,:)); %vClusters = vClusters(~isnan(vClusters));
iClusters = length(unique(vClusters));

%Specify locations in the results excel-file for the estimate. Note that
%beta locations need to be specified per cluster.
aLoc = ["2";"8";"3";"9";"4";"10";"5";"11";...    %domestic effects
        "14";"26";"15";"27";"16";"28";"17";"29";...%foreign effects from cluster 1 
        "20";"32";"21";"33";"22";"34";"23";"35"];  %foreign effects from cluster 2
    
iK   = length(aLoc);

%Specify time window around auction. iCentre specifies the location of the
%peak of the auction cycle (0 is auction day itself, positive numbers are days
%after the auction and negative are pre-auction.
%iDays_Pre and iDays_Post specify the time window before and after iCentre.
iCentre    = 0;
iDays_Pre  = 5;
iDays_Post = 5;
iWindow_len= iDays_Pre+iDays_Post;


% Include stuff for writing to Excel file (if using Mac), otherwise,
% xlwrite should be changed to xlswrite.
javaaddpath('../../xlwrite/poi_library/poi-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/poi-ooxml-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/poi-ooxml-schemas-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/xmlbeans-2.3.0.jar');
javaaddpath('../../xlwrite/poi_library/dom4j-1.6.1.jar');
javaaddpath('../../xlwrite/poi_library/stax-api-1.0.1.jar');

aOutput      = cell(2,1);
aOutput{1,1} = "E";
aOutput{2,1} = "F";

iLoc_QE     = find(vDates == datenum('10-05-2010', 'dd-mm-yyyy')); %Start SMP.
vDates_QE   = vDates(iLoc_QE:end-3); %yield data only goes until 27th dec. skip last 3.

for bCovariates = 0:1
    if bCovariates
        %Differences for EONIA, Log Differences for Indices.
        mX_covs = [aFinMarket{2,1}(iLoc_QE:end-3,2),log(aFinMarket{2,2}(iLoc_QE:end-3,2)),...
                   log(aFinMarket{2,3}(iLoc_QE:end-3,2)),log(aFinMarket{2,4}(iLoc_QE:end-3,2))];
        %mX_covs = [mX_covs(1,:);diff(mX_covs)]; 
        mX_covs = diff(mX_covs);
        mX_covs(:,2:4) = mX_covs(:,2:4)*100;
        mX_covs = [ones(2,4)*NaN;mX_covs(1:end-1,:)]; %first two lagged differences we don't observe; these observations are filtered out as we drop observations due to auction dummies.
    end
    for m=2:2
        disp('maturity')
        disp(m)
        
        aYields_change_m    = aYields_change(:,aSetCntry{m,1});
        aAUC_m              = aAUC(:,aSetCntry{m,1});
        aYields_Vola_m      = aYields_Vola(:,aSetCntry{m,1});
        aDH_interp_diff_m   = aDH_interp_blackout_diff_hat(:,aSetCntry{m,1});
        vClusters_m         = vClusters(1,aSetCntry{m,1});
        aGovDebt_m          = aGovDebt(:,aSetCntry{m,1});
        iN                  = numel(aYields_change_m(1,:));
        
        %------Dependent-Variable-----------------------------------------%
        vY_all =[];
        vID    =[];
        for i=1:iN
            vY_diff = aYields_change_m{m+1,i}(iLoc_QE:end,2)*100;                 %in bps
            iT_total= length(vY_diff);
            vY_all  = [vY_all,vY_diff];                             %#ok<AGROW>
            vID     = [vID,ones(iT_total,1)*i];                     %#ok<AGROW>
        end
        %-----------------------------------------------------------------%
        
        %------Independent-Variables--------------------------------------%
        mAUC        = zeros(iT_total,iN);
        mVola       = zeros(iT_total,iN);
        mDH         = zeros(iT_total,iN);
        mGovDebt    = zeros(iT_total,iN);
        for i=1:iN
            mAUC(:,i)       = aAUC_m{m+1,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2);
            mVola(:,i)      = aYields_Vola_m{m+1,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2);
            mDH(:,i)        = max(aDH_interp_diff_m{2,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2),0);
            mGovDebt(:,i)   = aGovDebt_m{2,i}( iLoc_QE:iLoc_QE+iT_total-1 ,2)/1000;
        end
        %mAUC_total    = sum(mAUC,2); %total auctions at time t, easy to create the foreign dummy.
        mAUC_total_C1 = sum(mAUC(:,vClusters_m == 1),2); %summed over the countries in cluster 1
        mAUC_total_C2 = sum(mAUC(:,vClusters_m == 2),2); %summed over the countries in cluster 2
        %-----------------------------------------------------------------%
        
        %------Panel-Regression-------------------------------------------%
        mX  =[];
        mDH = mDH./mGovDebt; 
        for i=1:iN
            mX_i            = zeros(iT_total,iN+iK*iWindow_len+3*iN);
            vIntercept      = zeros(iN,1);
            vIntercept(i)   = 1;
            for t=1+iDays_Post:iT_total-(iDays_Pre-1)
                vRange          = t+(iDays_Pre-1):-1:t-iDays_Post;
                vX_dom          = mAUC(vRange,i);
                vX_dom_VOL      = vX_dom.*mVola(t,i);
                vX_dom_DH       = vX_dom.*mDH(t,i);
                vX_dom_VOL_DH   = vX_dom.*mVola(t,i).*mDH(t,i);
                
                %vX_for       = (mAUC_total(vRange)-vX_dom);
                if vClusters_m(i) == 1
                    vX_for_C1_i = (mAUC_total_C1(vRange)-vX_dom);
                    vX_for_C2_i = (mAUC_total_C2(vRange));
                else %elseif vClusters_m(i) == 2
                    vX_for_C1_i = (mAUC_total_C1(vRange));
                    vX_for_C2_i = (mAUC_total_C2(vRange)-vX_dom);
                end
                
                %vX_for_C1_vol_i= vX_for_C1_i.*mVola(t,i);
                %vX_for_C2_vol_i= vX_for_C2_i.*mVola(t,i);
                vX_for_C1_vol_i= sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                                      mVola(t,setdiff(find(vClusters_m==1),i)),2);
                vX_for_C2_vol_i= sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                                      mVola(t,setdiff(find(vClusters_m==2),i)),2);
                
                %vX_for_DH    = sum(mAUC(vRange,1:end~=i).*mDH(t,1:end~=i),2);
                vX_for_C1_DH    = sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                                      mDH(t,setdiff(find(vClusters_m==1),i)),2);
                vX_for_C2_DH    = sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                                      mDH(t,setdiff(find(vClusters_m==2),i)),2);
                
                %vX_for_VOL_DH= sum(mAUC(vRange,1:end~=i).*mVola(t,i).*mDH(t,1:end~=i),2);
                vX_for_C1_VOL_DH = sum(mAUC(vRange,setdiff(find(vClusters_m==1),i)).*...
                                      mVola(t,setdiff(find(vClusters_m==1),i)).*...
                                      mDH(t,setdiff(find(vClusters_m==1),i)),2);
                vX_for_C2_VOL_DH = sum(mAUC(vRange,setdiff(find(vClusters_m==2),i)).*...
                                      mVola(t,setdiff(find(vClusters_m==2),i)).*...
                                      mDH(t,setdiff(find(vClusters_m==2),i)),2);
                                  
                
                mX_i(t,:)   = [vIntercept',...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_VOL',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_dom_VOL_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_vol_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C1_VOL_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_vol_i',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    zeros(1,(vClusters_m(i)-1)*iWindow_len),vX_for_C2_VOL_DH',zeros(1,(iClusters-vClusters_m(i))*iWindow_len),...
                    mVola(t,:),mDH(t,:),mVola(t,:).*mDH(t,:)];
            end
            if ~bCovariates
                mX = [mX;mX_i];                                 %#ok<AGROW>
            else
                mX = [mX;[mX_i,mX_covs]];                       %#ok<AGROW>
            end
        end
        %Only retain indices that contain data, i.e. cut out zeros.
        vIndex_i = [zeros(iDays_Post,1);ones(iT_total-iDays_Pre-iDays_Post+1,1);zeros((iDays_Pre-1),1)];
        vIndex   = repmat(vIndex_i , [iN 1]);
        
        vY_all = vY_all(find(vIndex));                             %#ok<*FNDSB>
        vID    = vID(find(vIndex));
        %mX     = mX(find(vIndex),:);
        mX     = mX(find(vIndex),setdiff((1:size(mX,2)),iN+[44,45,54,55,64,65,74,75,124,125,134,135,144,145,154,155,204,205,214,215,224,225,234,235])); %Get rid of AUC_t-1*DH_t-1, AUC_t, DH_t interactions (orthogonal)
        iT     = sum(vIndex_i);
        %-----------------------------------------------------------------%
        
        %------Panel-Results----------------------------------------------%
        vBeta  = lsqlin(mX, vY_all,[],[], [],[],[],[]);%lsqlin(mX, vY_all,[],[], mConstr, zeros(iK*(iWindow_len-1),1),[],[]);
        vResid = vY_all-mX*vBeta;
        
        dSSR    = vResid'*vResid;
        dSST    = (vY_all-mean(vY_all))'*(vY_all-mean(vY_all));
        dR2_adj = R2_adjusted( dSSR, dSST, iT*iN, size(mX,2) );
        
        %Born Breitung (2016)
        [dBB_Stat, dBB_PVal] = Autocorrelation_BornBreitung(iN,iT,vResid);
        
        %C.B. Hansen (2007)
        [mCov, ~, ~] = SandwichCovariance_Arellano(size(mX,2),iN,iT,vResid,mX); %size(mX,2) = iWindow_len*ik+iN
        %-----------------------------------------------------------------%
        
        %------Write-Results----------------------------------------------%
        mLoc1 =[iN+1,iN+1+(iDays_Pre-1)+iCentre;... %10
               iN+11,iN+11+(iDays_Pre-1)+iCentre;... %10
               iN+21,iN+21+(iDays_Pre-1)+iCentre;... %10
               iN+31,iN+31+(iDays_Pre-1)+iCentre;... %10 
               iN+41,iN+41+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+49,iN+49+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+57,iN+57+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+65,iN+65+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8 
               iN+73,iN+73+(iDays_Pre-1)+iCentre;... %10
               iN+83,iN+83+(iDays_Pre-1)+iCentre;... %10
               iN+93,iN+93+(iDays_Pre-1)+iCentre;... %10
               iN+103,iN+103+(iDays_Pre-1)+iCentre;... %10
               iN+113,iN+113+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+121,iN+121+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
               iN+129,iN+129+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+137,iN+137+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
               iN+145,iN+145+(iDays_Pre-1)+iCentre;... %10
               iN+155,iN+155+(iDays_Pre-1)+iCentre;... %10
               iN+165,iN+165+(iDays_Pre-1)+iCentre;... %10
               iN+175,iN+175+(iDays_Pre-1)+iCentre;... %10
               iN+185,iN+185+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+193,iN+193+(iDays_Pre-1-iDaysBlckOut)+iCentre;...%8
               iN+201,iN+201+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+207,iN+207+(iDays_Pre-1-iDaysBlckOut)+iCentre];%8
        mLoc2 = [iN+1+(iDays_Pre-1)+iCentre+1,iN+10;...%10
                iN+11+(iDays_Pre-1)+iCentre+1,iN+20;...%10
                iN+21+(iDays_Pre-1)+iCentre+1,iN+30;...%10
                iN+31+(iDays_Pre-1)+iCentre+1,iN+40;...%10 
                iN+41+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+48;...%8
                iN+49+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+56;...%8
                iN+57+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+64;...%8
                iN+65+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+72;...%8 
                iN+73+(iDays_Pre-1)+iCentre+1,iN+82;...%10
                iN+83+(iDays_Pre-1)+iCentre+1,iN+92;...%10
                iN+93+(iDays_Pre-1)+iCentre+1,iN+102;...%10
                iN+103+(iDays_Pre-1)+iCentre+1,iN+112;...%10
                iN+113+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+120;...%8
                iN+121+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+128;...%8
                iN+129+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+136;...%8
                iN+137+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+144;...%8
                iN+145+(iDays_Pre-1)+iCentre+1,iN+154;...%10
                iN+155+(iDays_Pre-1)+iCentre+1,iN+164;...%10
                iN+165+(iDays_Pre-1)+iCentre+1,iN+174;...%10
                iN+175+(iDays_Pre-1)+iCentre+1,iN+184;...%10
                iN+185+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+192;...%8
                iN+193+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+200;...%8
                iN+201+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+206;...%8
                iN+207+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+214];%8
        for k = 1:iK
            dEst= sum(vBeta(mLoc1(k,1):mLoc1(k,2))) - ...
                  sum(vBeta(mLoc2(k,1):mLoc2(k,2))) ;
            
            mR = zeros(1,length(vBeta));
            vR = 0;
            mR(mLoc1(k,1):mLoc1(k,2))   = ones(1,mLoc1(k,2)-mLoc1(k,1)+1);
            mR(mLoc2(k,1):mLoc2(k,2))   = -ones(1,mLoc2(k,2)-mLoc2(k,1)+1);
            
            [dF,dPVal] = FtestHansen(vBeta,mCov,mR,vR,iN,length(vR));
            sLoc       = aLoc(k);
            xlwrite("IV_results.xlsx", Write_significance(dEst, dPVal),...
                aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        end
        if bCovariates
            for i=1:4
                dEst        = vBeta(end-4+i);
                mR          = zeros(1,length(vBeta));
                vR          = 0;
                mR(end-4+i) = 1;
                [dF,dPVal]  = FtestHansen(vBeta,mCov,mR,vR,iN,length(vR));
                xlwrite("IV_results.xlsx", Write_significance(dEst, dPVal),...
                    aMaturities(m),aOutput{1+bCovariates,1}+(i+37));
            end
        end
        
        sLoc = "43";
        xlwrite("IV_results.xlsx", Write_significance(dBB_Stat, dBB_PVal),aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        sLoc = "44";
        xlwrite("IV_results.xlsx", dR2_adj,aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        sLoc = "45";
        xlwrite("IV_results.xlsx", iT,aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        sLoc = "46";
        xlwrite("IV_results.xlsx", iN,aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        %-----------------------------------------------------------------%
    end
end