function [mCov, mW, mQ] = SandwichCovariance_Arellano(iK,iN,iT,vResid,mX)
%This function implements the heteroskedasticity and (temporal) autocorrelation
%consistent covariance matrix of Arellano (1987) as in C.B Hansen (2007). 
%This matrix lies at the basis of the robust F-test designed by 
%C.B. Hansen (2007).
% 
% Inputs:
% - iK  = Number of parameters.
% - iN  = Number of individuals.
% - iT  = Number of time observations.
% - vResid = stacked Residuals (hence length = iT*iN)
% - mX  = stacked matrix of regressors (hence size = (iT*iN)*iK)

mW  = zeros(iK,iK);
mXX     = zeros(iK,iK);
for i=1:iN
    mW	= mW + 1/(iN*iT)*mX(iT*(i-1)+1:iT*i,:)'*vResid(iT*(i-1)+1:iT*i)*...
                vResid(iT*(i-1)+1:iT*i)'*mX(iT*(i-1)+1:iT*i,:);
    mXX = mXX + mX(iT*(i-1)+1:iT*i,:)'*mX(iT*(i-1)+1:iT*i,:);
end
mQ = mXX/(iN*iT);
mCov = mXX\(iN*iT*mW)/mXX;
end