function [dStat, dPVal] = Autocorrelation_BornBreitung(iN,iT,vResid)
%This functions implements the Born Breitung (2016) test for
%(first order) autocorrelation that is robust against heteroskedasticity,
%as opposed to the standard tests (Bhargava, Drukker, Inoue & Solomon).
%
% Inputs:
% - iN  = Number of individuals.
% - iT  = Number of time observations.
% - vResid = stacked Residuals (hence length = iT*iN)

mV_0 = zeros(iT-3,iT);
mV_1 = zeros(iT-3,iT);
for t = 1:iT-3
    mV_0(t,1:t)     = -1/(t+1);
    mV_0(t,t+1)     =  t/(t+1);
    mV_1(t,t+2)     =  (iT-(t+2))/(iT-(t+1));
    mV_1(t,t+3:end) =  -1/(iT-(t+1));
end
dZ_BB_s = 0;
dZ_BB_ss= 0;
for i=1:iN
    vZ_BB_i = vResid(iT*(i-1)+1:iT*i)'*mV_0'*mV_1*vResid(iT*(i-1)+1:iT*i);
    dZ_BB_s = dZ_BB_s + sum(vZ_BB_i);
    dZ_BB_ss= dZ_BB_ss+ sum(vZ_BB_i.^2);
end
dStat = dZ_BB_s/sqrt(dZ_BB_ss-1/iN*dZ_BB_s^2);
dPVal = 2*normcdf(dStat,0,1,'upper');
%slow
%vZ_BB           = (vResid'*kron(mV_0',eye(iN)))*(kron(mV_1,eye(iN))*vResid);
%dBB   = sum(vZ_BB)/sqrt(sum(vZ_BB.^2-1/iN*vZ_BB));
end