function [ aBeta, aSigma ] = FixedEffects_Bayes_Gibbs( mY,mX_intercept, ...
 mX_AUC,aX_AUC_VOL,mX_AUC_DH,aX_AUC_VOL_DH,...
 mX_FAUC,aX_FAUC_VOL,mX_FAUC_DH,aX_FAUC_VOL_DH,...
 mX_VOL_allC,mX_DH_allC,mX_VOL_DH_allC,iSim, ~, ~ )
%Gibbs sampler for a Fixed Effects Panel model.
%Based on the work Greenberg, Bayesian Econometrics Chapter 10.3 (model
%without random effects).

%mY, mX need to be specified correctly, i.e. intercept matrix needs to be
%included in mX (vectors of zeros(iT,1) concatenated. Basically, mIntercept
%= kron(eye(iN),ones(iT,1)).

%Initialisation-----------------------------------------------------------%
[iT, ~]    = size(mY);
iN          = 9;   %my is stacked!!.
iAucLength  = size(mX_AUC,2);

%initialize X matrix for speedup, only reallocate volatily involved variables.
%mX      = [mX_intercept,mX_AUC,aX_AUC_VOL(:,:,1),mX_AUC_DH,aX_AUC_VOL_DH(:,:,1),...
%           mX_FAUC,aX_FAUC_VOL(:,:,1),mX_FAUC_DH,aX_FAUC_VOL_DH(:,:,1)];

%initialize X matrix for speedup, only reallocate volatily involved variables.
mX      = [mX_intercept,mX_AUC,aX_AUC_VOL{:,1},mX_AUC_DH,aX_AUC_VOL_DH{:,1},...
           mX_FAUC,aX_FAUC_VOL{:,1},mX_FAUC_DH,aX_FAUC_VOL_DH{:,1},...
           mX_VOL_allC(:,:,1),mX_DH_allC,mX_VOL_DH_allC(:,:,1)];
iK      = size(mX,2);
disp(iK)
vLocs   = [(iN+iAucLength+1:iN+2*iAucLength)... %mX_vola
            iN+iAucLength*3-2+1:iN+iAucLength*4-4]; %volaxDH note minus 2 for DH windows
vLocs   =[vLocs,vLocs+iAucLength*4-4]; %same for foreign cylce, but shifted.
vLocs   = [vLocs,iK-26:iK-18,iK-8:iK];
fprintf('nonzero elements mX: %i\n', length(find(mX)))
iK = size(mX,2);

%Create arrays to store results.
%aBeta   = zeros(iK, iN, iSim);
aBeta   = zeros(iK, 1, iSim);
aSigma  = zeros(1, 1, iSim); %Homoskedastic
%-------------------------------------------------------------------------%


%Priors-------------------------------------------------------------------%
%Lambda, lower triangular matrix.
vB_0    = zeros(iK,1);
mB_0    = eye(iK)*10^1;
mB_0_inv= inv(mB_0);
%mBeta   = mvnrnd(vB_0, mB_0);

%Sigma
dAlpha_0= 2.2; %taken from Lopes & West (2004)
dDelta_0= 0.0455*dAlpha_0; 
dSigma  = 1/gamrnd( dAlpha_0/2 ,2/dDelta_0 ); 
%-------------------------------------------------------------------------%


%Gibbs--------------------------------------------------------------------%
dAlpha = dAlpha_0 + iT*iN;
mN_Beta= mvnrnd(zeros(iK,1), eye(iK), iSim);%pre sample all multivariate normals for speedup
for s=1:iSim
    %disp(s)
    %mX(:,vLocs) = [aX_AUC_VOL(:,:,s) ,aX_AUC_VOL_DH(:,:,s),...
    %             aX_FAUC_VOL(:,:,s),aX_FAUC_VOL_DH(:,:,s)];
    mX(:,vLocs) = [aX_AUC_VOL{:,s} ,aX_AUC_VOL_DH{:,s},...
                 aX_FAUC_VOL{:,s},aX_FAUC_VOL_DH{:,s},...
                 mX_VOL_allC(:,:,s),mX_VOL_DH_allC(:,:,s)];
    if mod(s,10^3)==0
        fprintf('Simulation %i\n',s)
    end
    
    %Update Beta
    %mXSinv= mX'*kron(eye(iN),1/dSigma*eye(iT)) %instead of summation over all i, take kronecker product of covariance matrix. (mX is stacked) 
    %However, is the same as below, due to homoskedasticity.
    %mXSinv  = mX'/dSigma; 
    
    mB_inv       = (mX'*mX)/dSigma + mB_0_inv;
    mB_inv_chol  = chol( mB_inv,'lower');
    %mB           = inv( (mX'*mX)/dSigma + mB_0_inv );   
    
    mB_inv_chol_inv = inv(mB_inv_chol);
    mB              = mB_inv_chol_inv'*mB_inv_chol_inv;
    %vB              = mB*(mXSinv*mY + mB_0_inv*vB_0);   %#ok<MINV>
    vB              = mB*( (mX'*mY)/dSigma + mB_0_inv*vB_0);   %#ok<MINV>
    %mBeta           = vB + mB_chol*mN_Beta(s,:)';
    mBeta           = vB + mB_inv_chol\mN_Beta(s,:)';
    aBeta(:,:,s)    = mBeta;
    
    %Update Sigma.
    mRes            = mY-mX*mBeta;
    dDelta          = dDelta_0 + mRes'*mRes;
    dSigma          = 1/gamrnd( dAlpha/2 ,2/dDelta );                       %inverted gamma with shape 1/b.
    aSigma(:,:,s)   = dSigma;    
end
%-------------------------------------------------------------------------%
end
