clear;clc;close all;
load("../../../Data/Yields/aYields_change.mat")
load("../../../Data/Market Volatility/Factor Analysis/aYields_Vola_Factor_individual.mat")
load("../../aAUC.mat")
load("../../aDH_interp_blackout_diff.mat")
load("../../../Data/Market Conditions/aFinMarket.mat")
load("../../aGDP.mat")


aYields_Vola = aYields_Vola_Factor_individual;
clear aYields_Vola_Factor_individual;

aMaturities = ["5Y";"10Y";"30Y"];
aCountries  = aAUC(1,:);
iN          = numel(aCountries);
vDates      = aAUC{2,1}(:,1);
iDaysBlckOut= 2; %pre auction and auction day.

%Specify which countries to include in the regression per maturity.
[~,aCountrySet,~]    = xlsread("../../../Data/CountrySetPerMaturity.xlsx","B1:Z3");
aSetCntry            = cell(3,1);
[~,aSetCntry{1,1},~] = intersect(aCountries,aCountrySet(1,:));   %5-Year maturity.
[~,aSetCntry{2,1},~] = intersect(aCountries,aCountrySet(2,:));   %10-Year maturity.
[~,aSetCntry{3,1},~] = intersect(aCountries,aCountrySet(3,:));   %30-Year maturity.

%Specify locations in the results excel-file for the estimate.
aLoc = ["2";"3";"4";"5";"9";"10";"11";"12"];
iK   = length(aLoc);

%Specify time window around auction. iCentre specifies the location of the
%peak of the auction cycle (0 is auction day itself, positive numbers are days
%after the auction and negative are pre-auction.
%iDays_Pre and iDays_Post specify the time window before and after iCentre.
iCentre    = 0;
iDays_Pre  = 5;
iDays_Post = 5;
iWindow_len= iDays_Pre+iDays_Post;


% % Include stuff for writing to Excel file (if using Mac), otherwise,
% xlwrite should be changed to xlswrite.
javaaddpath('../../xlwrite/poi_library/poi-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/poi-ooxml-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/poi-ooxml-schemas-3.8-20120326.jar');
javaaddpath('../../xlwrite/poi_library/xmlbeans-2.3.0.jar');
javaaddpath('../../xlwrite/poi_library/dom4j-1.6.1.jar');
javaaddpath('../../xlwrite/poi_library/stax-api-1.0.1.jar');

aOutput      = cell(2,1);
aOutput{1,1} = "N";
aOutput{2,1} = "O";

for bCovariates = 0:1
    if bCovariates
        %Differences for EONIA, Log Differences for Indices.
        mX_covs = [aFinMarket{2,1}(1:end-3,2),log(aFinMarket{2,2}(1:end-3,2)),...
                   log(aFinMarket{2,3}(1:end-3,2)),log(aFinMarket{2,4}(1:end-3,2))];
        %mX_covs = [mX_covs(1,:);diff(mX_covs)]; 
        mX_covs = diff(mX_covs);
        mX_covs(:,2:4) = mX_covs(:,2:4)*100;
        mX_covs = [ones(2,4)*NaN;mX_covs(1:end-1,:)]; %first two lagged differences we don't observe; these observations are filtered out as we drop observations due to auction dummies.
    end
    for m=2:2
        disp('maturity')
        disp(m)
        
        aYields_change_m    = aYields_change(:,aSetCntry{m,1});
        aAUC_m              = aAUC(:,aSetCntry{m,1});
        aYields_Vola_m      = aYields_Vola(:,aSetCntry{m,1});
        aDH_interp_diff_m   = aDH_interp_blackout_diff(:,aSetCntry{m,1});
        aGDP_m              = aGDP(:,aSetCntry{m,1});
        iN                  = numel(aYields_change_m(1,:));
        
        %------Dependent-Variable-----------------------------------------%
        vY_all =[];
        vID    =[];
        for i=1:iN
            vY_diff = aYields_change_m{m+1,i}(:,2)*100;                 %in bps
            iT_total= length(vY_diff);
            vY_all  = [vY_all,vY_diff];                             %#ok<AGROW>
            vID     = [vID,ones(iT_total,1)*i];                     %#ok<AGROW>
        end
        %-----------------------------------------------------------------%
        
        %------Independent-Variables--------------------------------------%
        mAUC        = zeros(iT_total,iN);
        mVola       = zeros(iT_total,iN);
        mDH         = zeros(iT_total,iN);     
        mGDP    = zeros(iT_total,iN);   
        for i=1:iN
            mAUC(:,i)  = aAUC_m{m+1,i}( 1:iT_total ,2);
            mVola(:,i) = aYields_Vola_m{m+1,i}( 1:iT_total ,2);
            mDH(:,i)   = max(aDH_interp_diff_m{2,i}( 1:iT_total ,2),0);
            mGDP(:,i)  = aGDP_m{2,i}( 1:iT_total ,2);%/1000
        end
        mAUC_total = sum(mAUC,2);
        %-----------------------------------------------------------------%
        
        %------Panel-Regression-------------------------------------------%
        mX =[];
        mDH_sc      = mDH./(mGDP/1000);
        for i=1:iN
            mX_i            = zeros(iT_total,iN+iK*iWindow_len+3*iN);
            vIntercept      = zeros(iN,1);
            vIntercept(i)   = 1;
            for t=1+iDays_Post:iT_total-(iDays_Pre-1)
                vRange       = t+(iDays_Pre-1):-1:t-iDays_Post;
                vX_dom       = mAUC(vRange,i);
                vX_dom_VOL   = vX_dom.*mVola(t,i);
                vX_dom_DH    = vX_dom.*mDH_sc(t,i);
                vX_dom_VOL_DH= vX_dom.*mDH_sc(t,i).*mVola(t,i);
                
                vX_for       = (mAUC_total(vRange)-vX_dom);
                vX_for_VOL   = sum(mAUC(vRange,1:end~=i).*mVola(t,1:end~=i),2);
                vX_for_DH    = sum(mAUC(vRange,1:end~=i).*mDH_sc(t,1:end~=i),2);
                vX_for_VOL_DH= sum(mAUC(vRange,1:end~=i).*mDH_sc(t,1:end~=i).*mVola(t,1:end~=i),2);
                
                mX_i(t,:)   = [vIntercept',...
                    vX_dom',vX_dom_VOL',vX_dom_DH',vX_dom_VOL_DH',...
                    vX_for',vX_for_VOL',vX_for_DH',vX_for_VOL_DH',...
                    mVola(t,:),mDH_sc(t,:),mVola(t,:).*mDH_sc(t,:)];
            end
            if ~bCovariates
                mX = [mX;mX_i];                                 %#ok<AGROW>
            else
                mX = [mX;[mX_i,mX_covs]];                       %#ok<AGROW>
            end
        end
        %Only retain indices that contain data, i.e. cut out zeros.
        vIndex_i = [zeros(iDays_Post,1);ones(iT_total-iDays_Pre-iDays_Post+1,1);zeros((iDays_Pre-1),1)];
        vIndex   = repmat(vIndex_i , [iN 1]);
        
        vY_all = vY_all(find(vIndex));                             %#ok<*FNDSB>
        vID    = vID(find(vIndex));
        %mX     = mX(find(vIndex),:);
        
        mX     = mX(find(vIndex),setdiff((1:size(mX,2)),iN+[24,25,34,35,64,65,74,75])); %Get rid of AUC_t-1*DH_t-1, AUC_t, DH_t interactions (orthogonal)
        iT     = sum(vIndex_i);
        %-----------------------------------------------------------------%
       
        
        %------Panel-Results----------------------------------------------%
        vBeta  = lsqlin(mX, vY_all,[],[], [],[],[],[]);%lsqlin(mX, vY_all,[],[], mConstr, zeros(iK*(iWindow_len-1),1),[],[]);
        vResid = vY_all-mX*vBeta;
        
        dSSR    = vResid'*vResid;
        dSST    = (vY_all-mean(vY_all))'*(vY_all-mean(vY_all));
        dR2_adj = R2_adjusted( dSSR, dSST, iT*iN, size(mX,2) );
        
        %Born Breitung (2016)
        [dBB_Stat, dBB_PVal] = Autocorrelation_BornBreitung(iN,iT,vResid);
        
        %C.B. Hansen (2007)
        [mCov, ~, ~] = SandwichCovariance_Arellano(size(mX,2),iN,iT,vResid,mX); %size(mX,2) = iWindow_len*ik+iN
        %-----------------------------------------------------------------%
        
        %------Write-Results----------------------------------------------%
        mLoc1 =[iN+1,iN+1+(iDays_Pre-1)+iCentre;... %10
               iN+11,iN+11+(iDays_Pre-1)+iCentre;... %10
               iN+21,iN+21+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+29,iN+29+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+37,iN+37+(iDays_Pre-1)+iCentre;... %10
               iN+47,iN+47+(iDays_Pre-1)+iCentre;... %10
               iN+57,iN+57+(iDays_Pre-1-iDaysBlckOut)+iCentre;... %8
               iN+65,iN+65+(iDays_Pre-1-iDaysBlckOut)+iCentre];
        mLoc2 = [iN+1+(iDays_Pre-1)+iCentre+1,iN+10;...%10
                iN+11+(iDays_Pre-1)+iCentre+1,iN+20;...%10
                iN+21+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+28;...%8
                iN+29+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+36;...%8
                iN+37+(iDays_Pre-1)+iCentre+1,iN+46;...%10
                iN+47+(iDays_Pre-1)+iCentre+1,iN+56;...%10
                iN+57+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+64;...%8
                iN+65+(iDays_Pre-1-iDaysBlckOut)+iCentre+1,iN+72];%8
        for k = 1:iK
            
            dEst= sum(vBeta(mLoc1(k,1):mLoc1(k,2))) - ...
                  sum(vBeta(mLoc2(k,1):mLoc2(k,2))) ;
            
            mR = zeros(1,length(vBeta));
            vR = 0;
            mR(mLoc1(k,1):mLoc1(k,2))   = ones(1,mLoc1(k,2)-mLoc1(k,1)+1);
            mR(mLoc2(k,1):mLoc2(k,2))   = -ones(1,mLoc2(k,2)-mLoc2(k,1)+1);
            
            [dF,dPVal] = FtestHansen(vBeta,mCov,mR,vR,iN,length(vR));
            sLoc       = aLoc(k);
            xlwrite("Panel_results.xlsx", Write_significance(dEst, dPVal),...
                aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
  
            if k == 1 || k == (iK/2 + 1) %Test the domestic and foreign cycle on symmetry
                dEst= sum(vBeta(mLoc1(k,1):mLoc1(k,2))) + ...
                sum(vBeta(mLoc2(k,1):mLoc2(k,2))) ; %Keep this format, not sum over the whole window, but two sums. If we alter centrality, this method accounts for it
                
                mR = zeros(1,length(vBeta));
                vR = 0;
                mR(mLoc1(k,1):mLoc1(k,2))   = ones(1,mLoc1(k,2)-mLoc1(k,1)+1);
                mR(mLoc2(k,1):mLoc2(k,2))     = ones(1,mLoc2(k,2)-mLoc2(k,1)+1);
                
                [dF,dPVal] = FtestHansen(vBeta,mCov,mR,vR,iN,length(vR));
                %Symmetry
                sLoc = 26; %excel sheet output location
                if k == (iK/2 + 1)
                    sLoc = 27;
                end
                xlwrite("Panel_results.xlsx", Write_significance(dEst, dPVal),...
                    aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
            end
        end
        if bCovariates
            for i=1:4
                dEst        = vBeta(end-4+i);
                mR          = zeros(1,length(vBeta));
                vR          = 0;
                mR(end-4+i) = 1;
                [dF,dPVal]  = FtestHansen(vBeta,mCov,mR,vR,iN,length(vR));
                xlwrite("Panel_results.xlsx", Write_significance(dEst, dPVal),...
                    aMaturities(m),aOutput{1+bCovariates,1}+(i+15));
            end
        end
        
        sLoc = "21";
        xlwrite("Panel_results.xlsx", Write_significance(dBB_Stat, dBB_PVal),aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        sLoc = "22";
        xlwrite("Panel_results.xlsx", dR2_adj,aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        sLoc = "23";
        xlwrite("Panel_results.xlsx", iT,aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        sLoc = "24";
        xlwrite("Panel_results.xlsx", iN,aMaturities(m),aOutput{1+bCovariates,1}+sLoc);
        %-----------------------------------------------------------------%
    end
end