function [dF,dPVal] = FtestHansen(vBeta,mCov,mR,vR,iN,iQ)
%This function implements the F-test designed by C.B. Hansen (2007).
%This F-test is developed to be robust against heteroskedasticity and
%temporal autocorrelation in a fixed effects panel model setting. This
%test delivers consistent inference even when iN is fixed and iT -> inf.

% Inputs:
% - vBeta = parameter vector.
% - mCov  = sandwich covariance estimate (following arellano 1987 as in Hansen 2007))
% - mR    = matrix with linear restrictions. size is iQxiK, where iQ is
%           number of restrictions and iK =length(vBeta) number of parameters
% - vR    = RHS of restriction (target value restriction) length is iQ.
% - iN  = Number of individuals.
% - iQ  = Number of restrictions

dF    = (mR*vBeta-vR)'/(mR*mCov*mR')*(mR*vBeta-vR);
dPVal = fcdf(dF/(iN*iQ/(iN-iQ)),iQ,iN-iQ,'upper');

 %dT_Hansen = sqrt(iN*iT)*(mR*vBeta-vR)/(sqrt(mR/mQ*mW/mQ*mR'));
end