%Function TheoreticalResultSecondaryMarket.m
%
%Author: J.J.M. van Spronsen
%
%Purpose: Compare baseline theoretical model results with that of the inclusion
%         of secondary market optimizing agent,by means of a symbolic expression.
%
%Inputs:    - 
%
%Outputs:   - Figure comparing the two models' dampening effects of the sovereign debt
%             purchases.


syms EF_do EF_dn EF_fo EF_fn eta W_d W_f W_g W_2;
syms s2_do s2_dn s2_fo s2_fn s_dodn s_dofo s_dofn s_dnfo s_dnfn s_fofn;
syms S_do S_dn S_fo S_fn;
syms alpha gamma;




EF = [EF_do;EF_dn;EF_fo;EF_fn];

S2 = gamma* [[s2_do ,s_dodn,s_dofo,s_dofn ];...
     [s_dodn,s2_dn ,s_dnfo, s_dnfn];...
     [s_dofo,s_dnfo,s2_fo ,s_fofn ];...
     [s_dofn,s_dnfn,s_fofn,s2_fn  ]];
S2_dd  = gamma* [[s2_do ,s_dodn];...
         [s_dodn,s2_dn]];
S2_ff  = gamma* [[s2_fo ,s_fofn];...
         [s_fofn,s2_fn ]];
S2_df  = gamma* [[s_dofo,s_dofn];...
         [s_dnfo, s_dnfn]];
S2_fd  = gamma* [[s_dofo,s_dnfo];...
         [s_dofn,s_dnfn]];
S2_2   = gamma*[[s2_do,s_dofo];...
         [s_dofo,s2_fo]];

beta_df = S2_df/S2_ff;
beta_fd = S2_fd/S2_dd;

A_inv  =[[inv((W_d+W_g)*eye(2)-(W_d*W_f)/(W_f+W_g)*(beta_df*beta_fd)),...
          W_f/(W_d+W_g)*beta_df/((W_f+W_g)*eye(2)-(W_d*W_f)/(W_d+W_g)*(beta_fd*beta_df))];...
         [W_d/(W_f+W_g)*beta_fd/((W_d+W_g)*eye(2)-(W_d*W_f)/(W_f+W_g)*(beta_df*beta_fd)),...
          inv((W_f+W_g)*eye(2)-(W_d*W_f)/(W_d+W_g)*(beta_fd*beta_df))]];

E_1 = W_2*[1;(s_dnfo*s_dofo-s_dodn*sqrt(s2_fo))/(s_dofo^2-sqrt(s2_do)*sqrt(s2_fo));0;(s_dofo*s_fofn-s_dofn*sqrt(s2_fo))/(s_dofo^2-sqrt(s2_do)*sqrt(s2_fo))];
E_1 = [E_1,zeros(4,3)];
E_2 = W_2*[0;(s_dnfo*sqrt(s2_do)-s_dodn*s_dofo)/(sqrt(s2_do)*sqrt(s2_fo)-s_dofo^2);1;(s_dofn*s_dofo-sqrt(s2_do)*s_fofn)/(s_dofo^2-sqrt(s2_do)*sqrt(s2_fo))];
E_2 = [zeros(4,2),E_2,zeros(4,1)];
g   = 1/(1+trace(A_inv*E_1));
h   = 1/(1+trace(A_inv*E_2));
Z   = eye(4) - h*E_2*A_inv + g*h*E_2*A_inv*E_1*A_inv - g*E_1*A_inv + g*h*E_1*A_inv*E_2*A_inv- g^2*h*E_1*A_inv*E_2*A_inv*E_1*A_inv;

%main equation P = EF - eta*A_inv*Z*S2*[ S_do; S_dn; S_fo; S_fn];

%Case 1: Two domestic bonds. Demand ECB: X_Edo=alpha*S_dn
P_orig  = EF - eta*A_inv*  S2*[ S_do-alpha*S_dn; S_dn; S_fo; S_fn];
P_exten = EF - eta*A_inv*Z*S2*[ S_do-alpha*S_dn; S_dn; S_fo; S_fn];

P_orig_1  = P_orig(1);
P_exten_1 = P_exten(1);

dP_orig_1  = diff(P_orig_1,S_dn);
dP_exten_1 = diff(P_exten_1,S_dn);

%calibration
dP_orig_1_calibr  = subs(dP_orig_1,[eta W_d W_f W_g W_2 s2_do s2_dn s2_fo s2_fn s_dodn s_dofo s_dofn s_dnfo s_dnfn s_fofn],...
                       [8    4   5   6  4   1.5   1.5   1.2   1.2   0.8     0.5    0.3    0.3    0.4    0.75]);
                   
dP_exten_1_calibr = subs(dP_exten_1,[eta W_d W_f W_g W_2 s2_do s2_dn s2_fo s2_fn s_dodn s_dofo s_dofn s_dnfo s_dnfn s_fofn],...
                       [8    4   5   6  4   1.5   1.5   1.2   1.2   0.8     0.5    0.3    0.3    0.4    0.75]);


alpha_cal = 0.0:0.05:0.5;
gamma_cal = 0.5:0.05:1.5;
mdP_orig  = zeros(length(alpha_cal),length(gamma_cal));
mdP_exten = zeros(length(alpha_cal),length(gamma_cal));
for i=1:length(alpha_cal)
    for j=1:length(gamma_cal)
        mdP_orig(i,j) = double(subs(dP_orig_1_calibr,[alpha,gamma],[alpha_cal(i),gamma_cal(j)]));  
        mdP_exten(i,j) = double(subs(dP_exten_1_calibr,[alpha,gamma],[alpha_cal(i),gamma_cal(j)]));  
    end
end

%figure
%surf(alpha_cal, gamma_cal, mdP_orig')
%title('Original Model')
%xlabel('\alpha')
%ylabel('\gamma')
%zlabel('\delta P{d,o} / \delta S_{d,n}')

%figure
%surf(alpha_cal, gamma_cal, mdP_exten')
%title('Extended Model')
%xlabel('\alpha')
%ylabel('\gamma')
%zlabel('\delta P{d,o} / \delta S_{d,n}')

figure
surf(alpha_cal, gamma_cal, mdP_orig', 'FaceColor','g', 'FaceAlpha',0.3, 'EdgeColor','none')
hold on
surf(alpha_cal, gamma_cal, mdP_exten', 'FaceColor','r', 'FaceAlpha',0.3, 'EdgeColor','none')
hold off
xlabel('\alpha')
ylabel('\gamma')
zlabel('\delta P_{d,o} / \delta S_{d,n}')
legend('Original','Extended', 'location', 'southeast')

%Case 2: One domestic bond, one foreign bond. Demand ECB: X_Efo=alpha*S_fn
%P = EF - eta*A_inv*Z*S2*[ S_do ; S_dn; S_fo-alpha*S_fn; S_fn];


