%Create individual volatility for country i by estimating the factor model on all
%countries except i, this ensures that the measure does not become
%endogenous.
clear; clc;
path_file   = '../../Yields/Yields.xlsx';

[~,aCountrySet,~] = xlsread("../../CountrySetPerMaturity.xlsx","B1:Z3");
[~,aSheets] = xlsfinfo(path_file);
aCountries  = aSheets;
iN          = numel(aCountries);
vDates      = x2mdate(xlsread(path_file,aSheets{1},'A2:A4954'));

%We stick to the calendar of the ECB.
vDates_ECB  = x2mdate(xlsread("../../Market Conditions/EONIA.xlsx",'A6:A4870'),0);
vDates_miss = setdiff(vDates,vDates_ECB);
vIndex      = zeros(length(vDates_miss),1);
for i=1:length(vDates_miss)
    vIndex(i) = find(vDates==vDates_miss(i));
end
vDates = vDates(setdiff(1:size(vDates,1),vIndex),:);

vMaturities = [5;10;30];
aYields_Vola_Factor_individual      = cell(4,iN);
aYields_Vola_Factor_individual(1,:) = aCountries;
aDevFromMean                        = cell(3,iN);

vN = zeros(3,1);
for m=1:length(vMaturities)
    mSqDevFromMean =[];
    aCountrySet_m  = aCountrySet(m,:);
    vN(m)          =numel(aCountrySet_m(~cellfun('isempty',aCountrySet_m)));
    for i = 1:vN(m)
        mData           = xlsread(path_file,aCountrySet{m,i},'B2:D4954');
        mData2          = mData(setdiff(1:size(mData,1),vIndex),:);
        vY_diff         = diff([mData(1,m);mData2(:,m)])*100;
        mSqDevFromMean  = [mSqDevFromMean, (vY_diff-mean(vY_diff)).^2]; %#ok<AGROW>
    end
    
    for i = 1:vN(m)
        [mLambda,~,~,~,vFactor] = factoran(sqrt(mSqDevFromMean(:,(1:vN(m))~=i)),1);
        iLoc = find(contains(aCountries,aCountrySet_m(i)));
        aYields_Vola_Factor_individual{m+1,iLoc} = [vDates,vFactor];
        aDevFromMean{m,i}                      = sqrt(mSqDevFromMean);
    end
end


figure 
subplot(3,1,1)
mPlot  = cell2mat(aYields_Vola_Factor_individual(2,:));
mPlot  = mPlot(:,(1:vN(1))*2); %get rid of time stamp.
plot(repmat(vDates,[1 vN(1)]), mPlot )
datetick('x','mm-yy')
legend(aCountrySet(1,:))
title('5Y')

subplot(3,1,2)
mPlot  = cell2mat(aYields_Vola_Factor_individual(3,:));
mPlot  = mPlot(:,(1:vN(2))*2); %get rid of time stamp.
plot(repmat(vDates,[1 vN(2)]), mPlot )
datetick('x','mm-yy')
legend(aCountrySet(2,:))
title('10Y')

subplot(3,1,3)
mPlot  = cell2mat(aYields_Vola_Factor_individual(4,:));
mPlot  = mPlot(:,(1:vN(3))*2); %get rid of time stamp.
plot(repmat(vDates,[1 vN(3)]), mPlot )
datetick('x','mm-yy')
legend(aCountrySet(3,:))
title('30Y')

figure
mPlot  = cell2mat(aYields_Vola_Factor_individual(3,:));
mPlot  = mPlot(:,(1:vN(2))*2); %get rid of time stamp.
for i=1:vN(2)
    subplot(5,2,i)
    plot(vDates,mPlot(:,i))
    datetick('x','mm-yy')
    xlim([vDates(1),vDates(end)]);
    title(aCountrySet{2,i})
end

disp('Correlation Market Volatilities')
disp(corr(mPlot))

fprintf('Latex table:\n\n')
fprintf('Country & Full Sample & Pre-APP & APP\\ \\ \n')
vIndx_preAPP = vDates<datenum('10-05-2010','dd-mm-yyyy');
vIndx_APP = vDates>=datenum('10-05-2010','dd-mm-yyyy');
for i=1:vN(2)
    fprintf('%s & Mean & %.2f & %.2f & %.2f\\\\ \n', string(aCountrySet{2,i}),...
            mean(mPlot(:,i),1),mean(mPlot(vIndx_preAPP,i),1),mean(mPlot(vIndx_APP,i),1))
    fprintf(' & St.Dev. & %.2f & %.2f & %.2f\\\\ \n',...
            std(mPlot(:,i),[],1),std(mPlot(vIndx_preAPP,i),[],1),std(mPlot(vIndx_APP,i),[],1))
end
%save('aYields_Vola_Factor_individual.mat', 'aYields_Vola_Factor_individual');