clear; clc;
path_file       = 'Yields.xlsx';
[~,aSheets]     = xlsfinfo(path_file);
iN              = numel(aSheets);

aYields         = cell(4,iN);
aYields_change  = cell(4,iN);
aCountries      = aSheets;
vMaturities     = [5;10;30];

vDates          = x2mdate(xlsread(path_file,aSheets{1},'A2:A4954'));

%We stick to the calendar of the ECB.
vDates_ECB  = x2mdate(xlsread("../Market Conditions/EONIA.xlsx",'A6:A4870'),0);
vDates_miss = setdiff(vDates,vDates_ECB);
vIndex      = zeros(length(vDates_miss),1);
for i=1:length(vDates_miss)
    vIndex(i) = find(vDates==vDates_miss(i));
end
%disp(vIndex)
vDates = vDates(setdiff(1:size(vDates,1),vIndex),:);

vSkip           = [4,10]; %Greece and Luxembourg (no data)

for i = 1:iN
    aYields{1,i}        = aCountries(i);
    aYields_change{1,i} = aCountries(i);
    %MATLAB does not read NAs. So for now skip Greece (EL) and LU.
    if ~ismember(i,vSkip)
    mData = xlsread(path_file,aSheets{i},'B2:D4954');
    mData2 = mData(setdiff(1:size(mData,1),vIndex),:);
    for m=1:length(vMaturities)
        aYields{m+1,i}          = [vDates,mData2(:,m)];
        aYields_change{m+1,i}   = [vDates,diff([mData(1,m);mData2(:,m)])]; %take difference with 31-dec-1998 as first observation in differences.
    end
    end
end

%vSkip = setdiff(1:iN,[1,6,12]);

figure('NumberTitle', 'off', 'Name', 'Yields');
subplot(3,1,1)
hold on
for i=1:iN
    if ~ismember(i,vSkip)
    plot(aYields{2,i}(:,1), aYields{2,i}(:,2) )
    datetick('x','mm-yy')
    end
end
legend(aCountries(setdiff(1:iN,vSkip)), 'location', 'northwest')
title('5Y')
hold off

subplot(3,1,2)
hold on
for i=1:iN
    if ~ismember(i,vSkip)
    plot(aYields{3,i}(:,1), aYields{3,i}(:,2) )
    datetick('x','mm-yy')
    end
end
legend(aCountries(setdiff(1:iN,vSkip)), 'location', 'northwest')
title('10Y')
hold off

subplot(3,1,3)
hold on
for i=1:iN
    if ~ismember(i,vSkip)
    plot(aYields{4,i}(:,1), aYields{4,i}(:,2) )
    datetick('x','mm-yy')
    end
end
legend(aCountries(setdiff(1:iN,vSkip)), 'location', 'northwest')
title('30Y')
hold off


figure('NumberTitle', 'off', 'Name', '\Delta Yields');
subplot(3,1,1)
hold on
for i=1:iN
    if ~ismember(i,vSkip)
    plot(aYields_change{2,i}(:,1), aYields_change{2,i}(:,2) )
    datetick('x','mm-yy')
    end
end
legend(aCountries(setdiff(1:iN,vSkip)), 'location', 'northwest')
title('5Y')
hold off

subplot(3,1,2)
hold on
for i=1:iN
    if ~ismember(i,vSkip)
    plot(aYields_change{3,i}(:,1), aYields_change{3,i}(:,2) )
    datetick('x','mm-yy')
    end
end
legend(aCountries(setdiff(1:iN,vSkip)), 'location', 'northwest')
title('10Y')
hold off

subplot(3,1,3)
hold on
for i=1:iN
    if ~ismember(i,vSkip)
    plot(aYields_change{4,i}(:,1), aYields_change{4,i}(:,2) )
    datetick('x','mm-yy')
    end
end
legend(aCountries(setdiff(1:iN,vSkip)), 'location', 'northwest')
title('30Y')
hold off


save('aYields.mat', 'aYields');
save('aYields_change.mat', 'aYields_change');